/***************************************************************************
                          chatmaster.cpp  -  description
                             -------------------
    begin                : Sat Jan 18 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "chatmaster.h"

#include <kdebug.h>

#include "../contact/contact.h"
#include "../network/chatinformation.h"
#include "../currentaccount.h"
#include "../kmessdebug.h"
#include "chatwindow.h"

// The constructor
ChatMaster::ChatMaster(QWidget *parent, const char *name )
 : QWidget(parent,name),
   initialized_(false)
{
}



// The destructor
ChatMaster::~ChatMaster()
{
  // Delete all the open chat windows
  for ( ChatWindow *chatWindow = chatWindows_.first(); chatWindow; chatWindow = chatWindows_.next() )
  {
    chatWindow->hide();
    chatWindow->close();
    delete chatWindow;
    chatWindows_.remove( chatWindow );
  }
  // Delete all the closed chat windows
  for ( ChatWindow *chatWindow = closedChatWindows_.first(); chatWindow; chatWindow = closedChatWindows_.next() )
  {
    delete chatWindow;
    closedChatWindows_.remove( chatWindow );
  }
#ifdef KMESSDEBUG_CHATMASTER
  kdDebug() << "DESTROYED ChatMaster " << endl;
#endif
}



// Close and remove a chat window
void ChatMaster::deleteChatWindow(ChatWindow *deletableChatWindow)
{
#ifdef KMESSDEBUG_CHATMASTER
  kdDebug() << "ChatMaster - Looking for chat window to close." << endl;
#endif
  bool found = false;
  for ( ChatWindow *chatWindow = chatWindows_.first(); chatWindow; chatWindow = chatWindows_.next() )
  {
    if ( chatWindow == deletableChatWindow )
    {
#ifdef KMESSDEBUG_CHATMASTER
      kdDebug() << "ChatMaster - Found it." << endl;
#endif
      found = true;
      break;
    }
  }
  // If the correct chat window was found...
  if ( found )
  {
#ifdef KMESSDEBUG_CHATMASTER
    kdDebug() << "ChatMaster - Closing the window." << endl;
#endif
    chatWindows_.remove( deletableChatWindow );
    deletableChatWindow->close();
    closedChatWindows_.append( deletableChatWindow );
  }
}



// The user has disconnected, so close all open chats
void ChatMaster::disconnected()
{
#ifdef KMESSDEBUG_CHATMASTER
    kdDebug() << "ChatMaster - Closing open chats." << endl;
#endif
  // Go through all the chat windows
  for ( ChatWindow *chatWindow = chatWindows_.first(); chatWindow; chatWindow = chatWindows_.next() )
  {
    // Close the chat window
    chatWindow->close();
    // Add it to the list of closed chat windows
    closedChatWindows_.append( chatWindow );
  }
  // Clear the chat windows
  chatWindows_.clear();
}



// Forward a new chat signal from a chat window
void ChatMaster::forwardNewChat(const Contact *contact, QString message, ChatWindow *chatWindow)
{
#ifdef KMESSTEST
  ASSERT( chatWindow != 0 );
#endif
#ifdef KMESSDEBUG_CHATMASTER
  kdDebug() << "ChatMaster - Forwarding the new chat signal." << endl;
#endif
  emit newChat( contact, message, chatWindow );
}



// Initialize the class
bool ChatMaster::initialize()
{
  if ( initialized_ )
  {
    kdDebug() << "ChatMaster already initialized." << endl;
    return false;
  }

  initialized_ = true;
  return true;
}



// Start a chat
void ChatMaster::startChat( ChatInformation *chatInfo )
{
#ifdef KMESSTEST
  ASSERT( chatInfo != 0 );
#endif
  ChatWindow *chatWindow;
  // Look through the chat windows for an exclusive chat with the contact given
  for ( chatWindow = chatWindows_.first(); chatWindow; chatWindow = chatWindows_.next() )
  {
    if ( chatWindow->isExclusiveChatWithContact( chatInfo->getContactHandle() ) )
    {
      // This chat window is already a chat with the contact, so reopen it.
      chatWindow->startChat( chatInfo );

      return;
    }
  }
  // Create a new chat window
#ifdef KMESSDEBUG_CHATMASTER
  kdDebug() << "ChatMaster - Creating a new chat window." << endl;
#endif
  chatWindow = new ChatWindow( this, "chatwindow" );
  connect( chatWindow, SIGNAL(         deleteMe( ChatWindow* ) ),
           this,       SLOT  ( deleteChatWindow( ChatWindow* ) ) );
  connect( chatWindow, SIGNAL(          newChat(const Contact*, QString, ChatWindow*) ),
           this,       SLOT  (   forwardNewChat(const Contact*, QString, ChatWindow*) ) );
  chatWindows_.append( chatWindow );
  if ( !chatWindow->initialize() )
  {
    kdDebug() << "ChatMaster - Couldn't initialize the new chat window." << endl;
    return;
  }
  chatWindow->startChat( chatInfo );
}
