/***************************************************************************
                          contact.h  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACT_H
#define CONTACT_H

#include <qwidget.h>

#include "contactbase.h"
#include "contactextension.h"

// Forward declarations
class ContactListViewItem;

/**A contact from the contact list.
  *@author Mike K. Bennett
  */

class Contact : public ContactBase
{
  Q_OBJECT

  public: // Public methods
    // The constructor
  	                     Contact(QString handle, QString friendlyName, QString list, QString groupId);
    // The constructor
  	                     Contact(QString handle, QString friendlyName, int list, QString groupId);
    // The destructor
  	                    ~Contact();
    // Check if this is a new contact
    bool                 checkIfContactAddedUser();
    // Return the path to the contact's picture
    QString              getContactPicturePath() const;
    // Return a pointer to the extension class
    ContactExtension    *getExtension() const;
    // Return the contact's friendly name
    QString              getFriendlyName() const;
    // Return a pointer to the contact's list view item
    ContactListViewItem* getItem();
    // Return a group ID based on the user's visibility and mode settings.
    QString              getMetaGroupId() const;
    // Return the group that the contact's in
    QString              getParentGroupId() const;
    // Return the contact's status
    QString              getStatus() const;
    // Return the contact's true friendly name, regardless of extension
    QString              getTrueFriendlyName() const;
    // Initialize the object
    bool                 initialize(ContactListViewItem *groupItem, const QString parentGroupId);
    // Whether or not the contact is allowed
    bool                 isAllowed() const;
    // Whether or not the contact is blocked
    bool                 isBlocked() const;
    // Whether or not the contact is on the friends list
    bool                 isFriend() const;
    // Whether or not the contact is on the reverse list
    bool                 isReverse() const;
    // Set whether or not the contact is allowed
    void                 setAllowed(bool allowed);
    // Set whether or not the contact is blocked
    void                 setBlocked(bool blocked);
    // Set whether or not the contact is on the friends list
    void                 setFriend(bool isFriend);
    // Change the contact's friendly name
    void                 setFriendlyName(QString newName);
    // Change the contact's normal group id
    void                 setGroupId(QString groupId);
    // Set whether or not the user is on a given list
    void                 setList(const QString list, bool isMember);
    // Set whether or not the user is on a given list
    void                 setList(int list);
    // Tell the contact what group it's in
    void                 setParentGroupId(const QString parentGroupId);
    // Set whether or not the contact is on the reverse list
    void                 setReverse(bool reverse);
    // Set the contact's status
    void                 setStatus(const QString status);
    // Update the visibilitiy of the item
    void                 updateItemVisibility();

  private: // Private methods
    // Update the item identifier and pixmap
    void                 updateItemIdentifierAndPixmap();

  private: // Private attributes
    // Whether or not the contact is "allowed"
    bool                 allowed_;
    // Whether or not the contact is blocked
    bool                 blocked_;
    // The extensions class
    ContactExtension    *extension_;
    // Whether or not the contact is a friend
    bool                 friend_;
    // The contact's group id
    QString             groupId_;
    // Whether or not the object has been initialized
    bool                 initialized_;
    // The contact item
    ContactListViewItem *item_;
    // The group that the contact's currently in
    QString             parentGroupId_;
    // Whether or not the contact is on the reverse list
    bool                 reverse_;
    // The contact's online status
    QString             status_;

  private slots: // Slots
    void                 emitChangedFriendlyName();
    // Forward the "changed picture" signal
    void                 forwardChangedPicture();

  signals: // Public signals
    // Signal that the contact may have changed list affiliation
    void                 changedList();
    // Signal that the contact is a new entry on the contact list
    void                 contactAddedUser(QString handle, QString friendlyName);
    // Signal if the contact goes offline
    void                 contactOffline(const Contact *contact);
    // Signal if the contact goes online
    void                 contactOnline(const Contact *contact);
};

#endif
