/***************************************************************************
                          contactextension.cpp  -  description
                             -------------------
    begin                : Fri 13 Dec 2002
    copyright            : (C) 2003 by Michael Curtis.
    email                : mdcurtis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contactextension.h"

#include <qdir.h>
#include <qfile.h>

#include <kconfig.h>
#include <kdebug.h>
#include <klocale.h>

#include "contact.h"
#include "../currentaccount.h"
#include "../kmessdebug.h"


// The constructor with specified contact handle
ContactExtension::ContactExtension(QString handle )
 : alternativeName_(""),
   notifyOnline_(true),
   pictureFile_(""),
   soundFile_(""),
   useAlternativeName_(false)
{
  config_ = kapp->config();
  CHECK_PTR(config_);

  loadContact( handle );
}


// The destructor
ContactExtension::~ContactExtension()
{
  saveContact();
}



// Read property of QString  alternativeName_.
const QString & ContactExtension::getAlternativeName() const
{
	return alternativeName_;
}



// Read property of QString  PictureFile_.
const QString ContactExtension::getContactPicturePath() const
{
  QString type = "pics", extension = "png";

  return guessFile( pictureFile_, type, extension);
}



// Read property of QString  SoundFile_.
const QString ContactExtension::getContactSoundPath() const
{
  QString type = "sounds", extension = "ogg";

  return guessFile( soundFile_, type, extension);
}



// Read property of bool notifyOnline_
bool ContactExtension::getNotifyOnline() const
{
	return notifyOnline_;
}



// Read property of bool useAlternativeName_.
bool ContactExtension::getUseAlternativeName() const
{
	return useAlternativeName_;
}



// Guesses a file location for contact "resources" such as pixmaps and sounds
const QString  ContactExtension::guessFile( const QString& file, QString& /*type*/, QString& extension) const
{
  QDir    homeDir;
  QFile   guessedFile;

  homeDir = QDir::home();

  if ( file == "" )
  {
//     guessedFile.setName( homeDir.path() + "/.kmess/" + type + "/" + shortHandle_ + "." + extension );
     guessedFile.setName( homeDir.path() + "/" + shortHandle_ + "." + extension );
  }
  else
  {
     guessedFile.setName( file );
  }

  if (guessedFile.exists())
  {
    return guessedFile.name();
  }
  else
  {
    return QString::null;
  }

}



// Load extension settings for a contact
void ContactExtension::loadContact( const QString& handle )
{
  QString groupName;

  handle_ = handle;

  groupName = "Contact_" + CurrentAccount::instance()->getHandle() + "/" + handle_;

#ifdef KMESSDEBUG_CONTACTEXTENSION
  kdDebug() << "COE - loadContact - finding group " << groupName << endl;
#endif

  if ( config_->hasGroup( groupName ) == true )
  {
#ifdef KMESSDEBUG_CONTACTEXTENSION
    kdDebug() << "COE - group exists" << endl;
#endif
    config_->setGroup( groupName );

    alternativeName_       =      config_->readEntry("alternativeName",          "");
    // We use the accessor function here because it handles the update, if required
    setUseAlternativeName  (  config_->readBoolEntry("useAlternativeName",    false) );

    notifyOnline_          =  config_->readBoolEntry("notifyOnline",           true);

    pictureFile_           =      config_->readEntry("pictureFile",              "");
    soundFile_             =      config_->readEntry("soundFile",                "");
  }

  // Get a version of the handle minus @whatever.com
  shortHandle_ = handle_;
  shortHandle_ = shortHandle_.left( shortHandle_.find("@") );

}



// Save extension settings for a contact
void ContactExtension::saveContact( void )
{
#ifdef KMESSDEBUG_CONTACTEXTENSION
    kdDebug() << "COE - saving data for contact " << handle_ << endl;
#endif
  config_->setGroup( "Contact_" + CurrentAccount::instance()->getHandle() + "/" + handle_ );

  config_->writeEntry( "useAlternativeName",      useAlternativeName_       );
  config_->writeEntry( "alternativeName",         alternativeName_          );
  config_->writeEntry( "notifyOnline",            notifyOnline_             );
  config_->writeEntry( "pictureFile",             pictureFile_              );
  config_->writeEntry( "soundFile",               soundFile_                );
}



// Write property of QString alternativeName_.
void ContactExtension::setAlternativeName( const QString& _newVal )
{
  if ( _newVal != "" )
  {
  	alternativeName_ = _newVal;

    if (getUseAlternativeName() == true) emit changedFriendlyName();
  }
}




// Write property of QString  pictureFile_.
void ContactExtension::setContactPicturePath( const QString & _newVal)
{
	pictureFile_ = _newVal;
  emit changedPicture();
}



// Write property of QString  soundFile_.
void ContactExtension::setContactSoundPath( const QString & _newVal)
{
	soundFile_ = _newVal;
}



// Write propety of bool notifyOnline_
void ContactExtension::setNotifyOnline( bool _newVal )
{
	notifyOnline_ = _newVal;
}



// Write property of bool useAlternativeName_
void ContactExtension::setUseAlternativeName( bool _newVal)
{
  bool needUpdate;

  needUpdate = ( useAlternativeName_ != _newVal);

	useAlternativeName_ = _newVal;

  if ( needUpdate == true ) emit changedFriendlyName();
}


