/***************************************************************************
                          contactlistview.cpp  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Michael Curtis
    email                : mdcurtis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contactlistview.h"

#include <qpainter.h>
#include <qpixmap.h>
#include <qobject.h>
#include <qwidget.h>

#include <kdebug.h>
#include <kstddirs.h>

#include "../specialgroups.h"
#include "contactdrag.h"
#include "contactlistviewitem.h"

// The oonstructor
ContactListView::ContactListView( QWidget *parent, const char *name, WFlags f)
 : QListView( parent, name, f )
{
  setShowToolTips( true );
}

ContactListView::~ContactListView()
{
}



// do we want this drop?
void ContactListView::contentsDropEvent( QDropEvent *event )
{
  if ( event->provides( ContactDragInfo::mimeType ) )
  {
    QListViewItem *item = itemAt( contentsToViewport( event->pos() ) );
    QListViewItem *parent;

    // Make sure we actually dropped it onto something that exists
    //  and that we were dragging in the first place
    if ( item && lastDragged_ )
    {
      // If the item has a parent, then use that.
      //  (we want the top level item that the item belongs to
      //    ... or if the item is the top level item, then use that)
      parent = item->parent();
      if ( !parent )
      {
        parent = item;
      }

      // tell the world
      emit moveGroupRequest( lastDragged_, (ContactListViewItem *) parent );

      // make sure we don't get spurious moves or anything like that
      lastDragged_ = 0;
 
      // We've accepted the event
      event->accept();
    }
  }

  QListView::contentsDropEvent( event );
}


// something being dragged has entered our widget
void ContactListView::contentsDragEnterEvent( QDragEnterEvent *event )
{
  if ( event->provides( ContactDragInfo::mimeType ) )
  {
    event->accept( true );
  }

  QListView::contentsDragEnterEvent( event );
}



// something being dragged has been moved in our widget
void ContactListView::contentsDragMoveEvent( QDragMoveEvent *event )
{
  if ( event->provides( ContactDragInfo::mimeType ) )
  {
    event->acceptAction( true );
  }

  QListView::contentsDragMoveEvent( event );
}



// creates and returns a drag object
QDragObject *ContactListView::dragObject()
{
  ContactDragObject *dragObject;

  dragObject = new ContactDragObject( this, "contactDragObject" );

  lastDragged_ = (ContactListViewItem *) selectedItem();

  return dragObject;
}



// returns the last item that was dragged
ContactListViewItem *ContactListView::getLastItemDragged()
{
  ContactListViewItem *lastDragged;

  // take a copy of the pointer...
  lastDragged = lastDragged_;

  // ...because we want to zero it...
  lastDragged_ = 0;

  // ...before finishing and returning what it use to point to
  return lastDragged;
}

// sets the background pixmap
void ContactListView::setBackground(QPixmap &background)
{
	backgroundPixmap_ = background;
	setPaletteBackgroundPixmap( background );
//	unsetPalette();
//	resizeEvent(0);
}

// called if the widget is resized
void ContactListView::resizeEvent(QResizeEvent * resizeEvent)
{
/*	if(!backgroundPixmap_.isNull())
	{
		QPixmap     pixmap(size());
		QPainter    painter(&pixmap);

		painter.fillRect(0, 0, width(), height(), QColor("white"));
		painter.drawPixmap((width() - backgroundPixmap_.width()) / 2, (height() - backgroundPixmap_.height()) / 2,
		                   backgroundPixmap_);

		setPaletteBackgroundPixmap(pixmap);
	}*/
  QScrollView::resizeEvent( resizeEvent );
}
