/***************************************************************************
                          contactlistview.h  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Michael Curtis
    email                : mdcurtis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACTLISTVIEW_H
#define CONTACTLISTVIEW_H

#include <qlistview.h>
#include <qpixmap.h>
#include <qwidget.h>

// Forward declarations
class ContactDragObject;
class ContactListViewItem;

/**The contactlistview is a subclass of qlistview, in order to properly handle
   drag and drop
  *@author Michael Curtis
  */

class ContactListView : public QListView
{
  Q_OBJECT

  public:
    // The constructor
    ContactListView( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    // The destructor
    ~ContactListView();

    // returns the last item that was dragged
    ContactListViewItem *getLastItemDragged();

    // sets the background pixmap
    void setBackground(QPixmap &background);

  protected:
    // creates and returns a drag object
    QDragObject *dragObject();
    // do we want this drop?
    void contentsDropEvent( QDropEvent *event );
    // something being dragged has entered our widget
    void contentsDragEnterEvent( QDragEnterEvent *event );
    // something being dragged has been moved in our widget
    void contentsDragMoveEvent( QDragMoveEvent *event );
    // called if the widget is resized
    void resizeEvent ( QResizeEvent *event );
  private: // private attributes
    ContactListViewItem *lastDragged_;
    // The original Pixmap used for the background
    QPixmap              backgroundPixmap_;

  signals:
    // An item was dragged from one group to another
    void moveGroupRequest( ContactListViewItem *contactMoving, ContactListViewItem *destinationGroup );

};

#endif
