/***************************************************************************
                          contactlistviewitem.cpp  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contactlistviewitem.h"

#include <qcolor.h>
#include <qmime.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qregexp.h>
#include <qsimplerichtext.h>

#include <kdebug.h>
#include <kiconloader.h>
#include <kstandarddirs.h>

#include "../emoticoncollection.h"

// only turn on if you want the program to slow down ;)
//#define KMESSDEBUG_CONTACTLISTVIEWITEM

// The constructor
ContactListViewItem::ContactListViewItem(QListView *parent)
 : QListViewItem( parent ),
   italicIdentifier_(false),
   onlineStatus_(false),
   richText_(0),
   sortIndex_(0),
   widthText_(0)
{
  // Do general construction
  construct();
}



// The constructor
ContactListViewItem::ContactListViewItem(QListViewItem *parent)
 : QListViewItem( parent ),
   italicIdentifier_(false),
   onlineStatus_(false),
   richText_(0),
   sortIndex_(0),
   widthText_(0)
{
  // Do general construction
  construct();
}



// The destructor
ContactListViewItem::~ContactListViewItem()
{
  if (  richText_ != 0 ) delete  richText_;
  if ( widthText_ != 0 ) delete widthText_;
}




// Compare
int ContactListViewItem::compare( QListViewItem *item, int, bool ) const
{
  int first, second, third, result;

  ContactListViewItem *otherItem = (ContactListViewItem *) item;

  first = getSortIndex() - otherItem->getSortIndex();
  second = otherItem->getOnlineStatus() - getOnlineStatus();
  third = getIdentifier().lower().localeAwareCompare( otherItem->getIdentifier().lower() );

  if ( first != 0 )
  {
    result = first;
  }
  else
  {
    if ( second != 0 )
    {
      result = second;
    }
    else
    {
      result = third;
    }
  }

  if ( result < 0 ) result = -1;
  if ( result > 0 ) result = 1;

  return result;
}



// Do general construction
void ContactListViewItem::construct()
{
  emoticons_ = EmoticonCollection::instance();

  setMultiLinesEnabled( false );
  recreateRichText();
}




// Return the identifier (primarily used for sorting)
const QString& ContactListViewItem::getIdentifier() const
{
  return identifier_;
}



// Get whether the contact is online or not
unsigned int ContactListViewItem::getOnlineStatus() const
{
  return onlineStatus_;
}



// Get the sort index
unsigned int ContactListViewItem::getSortIndex() const
{
  return sortIndex_;
}



// Paint a cell (column 0, always) - this allows us emoticons, through QSimpleRichText
void ContactListViewItem::paintCell( QPainter * painter, const QColorGroup & colourGroup, int column, int width, int align )
{
 if ( column == 0)
  {
    QBrush *paper;
    QPalette palette;
    QRect itemRectangle;

    palette = listView()->viewport()->palette();

    paper = 0;

    itemRectangle = listView()->itemRect( this );
    itemRectangle.setX( listView()->treeStepSize() * depth() );

    listView()->viewport()->erase( itemRectangle );

    if ( isSelected() )
    {
      paper = new QBrush( palette.color( QPalette::Active, QColorGroup::Highlight), Dense3Pattern );
    }

    richText_->draw( painter, listView()->itemMargin(), 0, QRect( 0, 0, width, height() ), colourGroup, paper );
    setHeight( richText_->height() );
    widthChanged( 0 );
  }
  else
  {
    QListViewItem::paintCell( painter, colourGroup, column, width, align );
  }
}



// Take HTML out of the name so it will show properly
void ContactListViewItem::parseHtmlChars( QString &text )
{
  text = text.replace( QRegExp("&" ), "&amp;"        );
  text = text.replace( QRegExp("<" ), "&lt;"         );
  text = text.replace( QRegExp(">" ), "&gt;"         );
  text = text.replace( QRegExp("::"), "&3A:"         );
  text = text.replace( QRegExp("  "), "&nbsp;&nbsp;" );
}



// Recreate the rich text
void ContactListViewItem::recreateRichText()
{
#ifdef KMESSDEBUG_CONTACTLISTVIEWITEM
  if ( isVisible() )
  {
    kdDebug() << "ContactListViewItem" << endl << "     Set the item text to \"" << text_ << "\" (length = " << text_.length() << ")." << endl;
  }
#endif
  // Delete the existing rich text
  if ( richText_ != 0 )
  {
    delete richText_;
    richText_ = 0;
  }

  // Delete the existing width text
  if ( widthText_ != 0 )
  {
    delete widthText_;
    widthText_ = 0;
  }

  // Create a new rich text object
  richText_ = new QSimpleRichText( "<qt><nobr>" + text_ + "</nobr></qt>", listView()->viewport()->font(), QString::null, 0/*, mimeFactory_*/ );

  // Creat a rich text object with 8bit characters to get the correct size
  QCString t = text_.local8Bit();
  QSimpleRichText *widthText_ = new QSimpleRichText( "<qt><nobr>" + t + "</nobr></qt>", listView()->viewport()->font(), QString::null, 0/*, mimeFactory_*/ );

#ifdef KMESSDEBUG_CONTACTLISTVIEWITEM
  if ( isVisible() )
  {
    kdDebug() << "     The rich text wants to use a width of " << richText_->width() << ".  Width used is " << richText_->widthUsed() << "." << endl;
    kdDebug() << "     test text width used = " << test->widthUsed() << "." << endl;
  }
#endif

  //richText_->adjustSize();
  // For now, give the contact a ridiculous width
//  richText_->setWidth(1000);
  richText_->setWidth( widthText_->widthUsed() );
#ifdef KMESSDEBUG_CONTACTLISTVIEWITEM
  if ( isVisible() )
  {
    kdDebug() << "     Width is now " << richText_->width() << "." << endl;
  }
#endif
}



// Set the item's icon
void ContactListViewItem::setIcon(const QString& iconName)
{
#ifdef KMESSDEBUG_CONTACTLISTVIEWITEM
//  kdDebug() << "ContactListViewItem - Setting the icon to \"" << iconName << "\"." << endl;
#endif
  KIconLoader     iconLoader;
  KStandardDirs   directory;
  QString         iconPath;

  // Find the path to the given icon
  iconPath = directory.findResource( "data", "kmess/pics/" + iconName + ".png" );

  // Not found?  Try KDE standard icons
  if ( iconPath.isEmpty() )
  {
    iconPath = iconLoader.iconPath( iconName, KIcon::Small, false );
  }

#ifdef KMESSDEBUG_CONTACTLISTVIEWITEM
//  kdDebug() << "ContactListViewItem - Icon found at \"" << iconPath << "\"." << endl;
#endif
  // Create an item <img> tag from the icon path
  iconTag_ = "<img align=left src=\"" + iconPath + "\">";

  updateItemText();
}



// Set the item's identifier
void ContactListViewItem::setIdentifier(const QString& identifier)
{
  identifier_ = identifier;
  parseHtmlChars( identifier_ );
  updateItemText();
}



// Set whether the text is italic or not
void ContactListViewItem::setItalic( bool italic )
{
  italicIdentifier_ = italic;

  updateItemText();
}



// Set whether the contact is online or not
void ContactListViewItem::setOnlineStatus( unsigned int online )
{
  onlineStatus_ = online;
}




// Set the sort index
void ContactListViewItem::setSortIndex( unsigned int sortIndex )
{
  sortIndex_ = sortIndex;
}



// Set the item's status identifier
void ContactListViewItem::setStatusIdentifier(const QString& statusIdentifier)
{
  statusIdentifier_ = statusIdentifier;
  updateItemText();
}



// dummy setText method
void ContactListViewItem::setText( int , const QString &)
{
  return;
}


// Update the item's text
void ContactListViewItem::updateItemText()
{
  text_ = iconTag_;

  if ( italicIdentifier_ )
  {
    text_ += "<i>";
  }

  text_ += identifier_;
  
  if ( italicIdentifier_ )
  {
    text_ += "</i>";
  }

  emoticons_->parseEmoticons( text_, true );

  if ( !statusIdentifier_.isEmpty() )
  {
    text_ += "  (" + statusIdentifier_ + ")";
  }

  recreateRichText();
}



// returns the width of the text in the column
int ContactListViewItem::width ( const QFontMetrics & fm, const QListView * lv, int c ) const
{
  if ( c == 0)
  {
    return richText_->width();
  }

  return QListViewItem::width( fm, lv, c );
}
