/***************************************************************************
                          contactlistviewitem.h  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACTLISTVIEWITEM_H
#define CONTACTLISTVIEWITEM_H

#include <qlistview.h>

// Forward declarations
class EmoticonCollection;
class QSimpleRichText;
class QString;

/**
  *@author Mike K. Bennett
  */

class ContactListViewItem : public QListViewItem
{
  public:
    // The constructor
  	                     ContactListViewItem(QListView *parent);
    // The constructor
  	                     ContactListViewItem(QListViewItem *parent);
    // The destructor
  	                    ~ContactListViewItem();
    // Compare
    virtual int          compare( QListViewItem *item, int column, bool ascending ) const;
    // Return the identifier (primarily used for sorting)
    const QString &      getIdentifier() const;
    // Get whether the contact is online or not
    unsigned int         getOnlineStatus() const;
    // Get the sort index
    unsigned int         getSortIndex() const;
    // Paint a cell (column 0, always) - this allows us emoticons, through QSimpleRichText
    void                 paintCell( QPainter * painter, const QColorGroup & colourGroup, int column, int width, int align );
    // Set the item's icon
    void                 setIcon(const QString& iconName);
    // Set the item's identifier
    void                 setIdentifier(const QString& identifier);
    // Set whether the text is italic or not
    void                 setItalic( bool italic );
    // Set whether the contact is online or not
    void                 setOnlineStatus( unsigned int online );
    // Set the sort index
    void                 setSortIndex( unsigned int sortIndex );
    // Set the item's status identifier
    void                 setStatusIdentifier(const QString& statusIdentifier);
    // dummy setText method
    void                 setText( int column, const QString &text);

  private: // Private methods
    // Do general construction
    void                 construct();
    // Take HTML out of the name so it will show properly
    void                 parseHtmlChars( QString &text );
    // Recreate the rich text
    void                 recreateRichText();
    // Update the item's text
    void                 updateItemText();
    // returns the width of the text in the column
    int                  width ( const QFontMetrics & fm, const QListView * lv, int c ) const;

  private: // Private attributes
    // The meoticon parser
    EmoticonCollection  *emoticons_;
    // The url tag t othe contact or group's icon
    QString              iconTag_;
    // The contact or group's identifier
    QString              identifier_;
    // Whether the identifier is italicised
    bool                 italicIdentifier_;
    // Whether the contact is online or offline
    unsigned int         onlineStatus_;
    // The rich text painter
    QSimpleRichText     *richText_;
    // The sort index, overrides other forms of comparison
    unsigned int         sortIndex_;
    // The contact or group's status identifier
    QString              statusIdentifier_;
    // The item text
    QString              text_;
    // The rich text painter used to get the real width
    QSimpleRichText     *widthText_;
};

#endif
