/***************************************************************************
                          group.h  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GROUP_H
#define GROUP_H

#include <qwidget.h>

// Forward declarations
class ContactListViewItem;
class KAction;
class KActionMenu;
class KConfig;
class QListView;
class QListViewItem;

/**A group from the user's contact list.
  *@author Mike K. Bennett
  */

class Group : public QWidget
{
  Q_OBJECT

  public: // Public methods
    // The constructor
  	                     Group(QString id, QString name, QListView *contactListView);
    // The destructor
  	                    ~Group();
    // Return the group ID
    const QString&       getId() const;
    // Return the group item
    ContactListViewItem* getItem();
    // Return the group name
    const QString&       getName() const;
    // Return the sort value
    const int&           getSortPosition() const;
    // Plug the group action into the given popup menu
    void                 plugAction(KActionMenu *popup);
    // Save group properties
    void                 saveProperties(KConfig *config);
    // Change the group name
    void                 setName(const QString& newName);
    // Change the sort position
    void                 setSortPosition(const int& sortPosition);
    // Unplug the group from the popup menu
    void                 unplugAction(KActionMenu *popup);
    // Update the item identifier
    void                 updateItemIdentifier();
    // Update the visibilitiy of the item
    void                 updateItemVisibility();

  private: // Private methods
    // Read in group properties
    void                 readProperties();

  private slots: // Private slots
    // The action was activated
    void                 actionActivated();

  private: // Private attributes
    // An action used in the "move contact to group" menu
    KAction             *action_;
    // The group id
    QString              id_;
    // The list view item representing the group
    ContactListViewItem *item_;
    // The group name
    QString              name_;
    // The group's sort value
    int                  sortPosition_;

  signals: // Public signals
    // Signal that the action was selected and pass along this group's ID
    void                 actionSelected(QString id);

};

#endif
