/***************************************************************************
                          contactnamedialog.cpp  -  description
                             -------------------
    begin                : Mon Nov 11 2002
    copyright            : (C) 2002 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contactnamedialog.h"

#include <qlabel.h>
#include <qlayout.h>

#include <kdebug.h>
#include <klineedit.h>
#include <klocale.h>

// The constructor
ContactNameDialog::ContactNameDialog(QWidget *parent, const char *name )
	: KDialogBase( parent, name, false, i18n("Change this person's name"), Ok | Cancel, Ok, true )
{
  setInitialSize( configDialogSize("ContactNameDialog") );

  QWidget *page = new QWidget( this );
  setMainWidget( page );

  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  QLabel *infoLabel = new QLabel( i18n("Enter the name you would prefer to have used for this person:"), page, "info" );
  topLayout->addWidget( infoLabel );

  friendlyNameEdit_ = new KLineEdit( page, "handleedit" );
  friendlyNameEdit_->setMinimumWidth( fontMetrics().maxWidth()*20 );
  topLayout->addWidget( friendlyNameEdit_ );

  topLayout->addStretch( 10 );

  adjustSize();
}



// The destructor
ContactNameDialog::~ContactNameDialog()
{
}



// Show the dialog and, if OK is pressed, change the Contact name to what
//  the user has selected.
bool ContactNameDialog::launch(QString &contactName)
{
  // Set whether the "ok" button was pressed to false
  ok_ = false;
  // Set the given Contact name to the text edit as the default Contact name
  friendlyNameEdit_->setText( contactName );
  // Exec the form (to show it modally).
  exec();
  // If OK was pressed...
  if ( ok_ )
  {
    // If the text edit isn't blank or unchanged
    if (    ( friendlyNameEdit_->text() != "" )
         && ( friendlyNameEdit_->text() != contactName ) )
    {
      // Get the Contact name from the text edit and store it
      contactName = friendlyNameEdit_->text();
    }
    else
    {
      // If the text is blank, ok should be returned as false, since we
      //  won't change the Contact name.
      ok_ = false;
    }
  }
  return ok_;
}



// The Cancel button was pressed.
void ContactNameDialog::slotCancel()
{
  saveDialogSize("ContactNameDialog");
  reject();
}



// The OK button was pressed.
void ContactNameDialog::slotOk()
{
  saveDialogSize("ContactNameDialog");
  ok_ = true;
  accept();
}
