/***************************************************************************
                          groupnamedialog.cpp  -  description
                             -------------------
    begin                : Sun Apr 14 2002
    copyright            : (C) 2002 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "groupnamedialog.h"

#include <qlabel.h>
#include <qlayout.h>

#include <kdebug.h>
#include <klineedit.h>
#include <klocale.h>

GroupNameDialog::GroupNameDialog(QWidget *parent, const char *name )
	: KDialogBase( parent, name, false, i18n("Group Name"), Ok | Cancel, Ok, true )
{
  setInitialSize( configDialogSize("GroupNameDialog") );

  QWidget *page = new QWidget( this );
  setMainWidget( page );

  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  QLabel *infoLabel = new QLabel( i18n("Enter the new name for this group:"), page, "info" );
  topLayout->addWidget( infoLabel );

  nameEdit_ = new KLineEdit( page, "nameEdit" );
  nameEdit_->setMinimumWidth( fontMetrics().maxWidth()*20 );
  topLayout->addWidget( nameEdit_ );

  topLayout->addStretch( 10 );

  adjustSize();
}



GroupNameDialog::~GroupNameDialog()
{
}



// Show the dialog and, if OK is pressed, change the group name to what
//  the user has selected.
bool GroupNameDialog::launch(QString &groupName)
{
  // Set whether the "ok" button was pressed to false
  ok_ = false;
  // Set the given group name to the text edit as the default group name
  nameEdit_->setText( groupName );
  // Exec the form (to show it modally).
  exec();
  // If OK was pressed...
  if ( ok_ )
  {
    // If the text edit isn't blank
    if ( nameEdit_->text() != "" )
    {
      // Get the group name from the text edit and store it
      groupName = nameEdit_->text();
    }
    else
    {
      // If the text is blank, ok should be returned as false, since we
      //  won't change the group name.
      ok_ = false;
    }
  }
  return ok_;
}



// The Cancel button was pressed.
void GroupNameDialog::slotCancel()
{
  saveDialogSize("GroupNameDialog");
  reject();
}



// The OK button was pressed.
void GroupNameDialog::slotOk()
{
  saveDialogSize("GroupNameDialog");
  ok_ = true;
  accept();
}
