/***************************************************************************
                          application.cpp  -  description
                             -------------------
    begin                : Thu Mar 20 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "application.h"

#include <stdlib.h>

#include <kdebug.h>
#include <klocale.h>

#include "../../kmessdebug.h"
#include "../mimemessage.h"
#include "ipdialog.h"
                   
// The constructor
Application::Application(const QString& localIp)
 : QObject(),
   localIp_(localIp),
   userStartedApp_(true)
{
}



// The destructor
Application::~Application()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "DESTROYED Application" << endl;
#endif
}



// The contact cancelled the application
void Application::contactCancelled()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - contact cancelled." << endl;
#endif
  QString html;

  // Create a cancel message for the user.
  html  = "<font color=purple>";
  html += i18n("The contact has cancelled the invitation.");
  html += "</font>";
  emit messageToUser( html );

  // Signal that this receive transfer should be deleted.
  emit deleteMe( getCookie() );
}



// The contact rejected the application
void Application::contactRejected()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - contact rejected." << endl;
#endif
  QString html;

  // Create a cancel message for the user.
  html  = "<font color=purple>" + i18n("The contact has rejected the invitation.") + "</font>";
  emit messageToUser( html );

  // Signal that this receive transfer should be deleted.
  emit deleteMe( getCookie() );
}



// Step one of a contact-started chat: the contact invites the user
void Application::contactStarted1_ContactInvitesUser(const MimeMessage& /*message*/)
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - contactStarted1_ContactInvitesUser" << endl;
#endif
}



// Step two of a contact-started chat: the user accepts
void Application::contactStarted2_UserAccepts()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - contactStarted2_UserAccepts" << endl;
#endif
}



// Step three of a contact-started chat: the contact confirms the accept
void Application::contactStarted3_ContactConfirmsAccept(const MimeMessage& /*message*/)
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - contactStarted3_ContactConfirmsAccept" << endl;
#endif
}



// Free a used port
void Application::freePort(const int& port)
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - Freeing port " << port << "." << endl;
#endif
}



// Generate a random file or authorization cookie.
QString Application::generateCookie() const
{
  int     number, maxNumber;
  QString cookie;

  // The maximum size for a cookie is supposedly 2^32 - 1, though I've
  //  never seen one go higher than 100,000.
  maxNumber = 1048575;
  // Get a random number in the given range.
  number = rand()%maxNumber;
  // Convert the number to a QCString
  cookie.sprintf("%d", number);
  // Return the cookie
  return cookie;
}



// Generate a random session ID
QString Application::generateSessionId() const
{
  QString sessionId = "{6672F94C-45BF-11D7-B4AE-00010A1008DF}";

#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - Generated session ID " << sessionId << "." << endl;
#endif

  return sessionId;
}



// Return an available port
int Application::getAvailablePort()
{
  return 6891; // This is a bad default to return
}



// Return the application's identifying cookie
const QString& Application::getCookie() const
{
  return cookie_;
}



// Return the local IP address, confirming it with the user if neccessary
const QString& Application::getLocalIp(bool confirm)
{
  IpDialog *ipDialog;
  QString   specifiedIp;
  bool      useSpecifiedIp;

  // Confirm the local IP with the user
  if ( confirm )
  {
    // Get the local IP address
    specifiedIp = localIp_;
    // Create an IP Dialog
    ipDialog = new IpDialog (0, "ipdialog");
    // Confirm that IP with the user
    useSpecifiedIp = ipDialog->getIpAddress( specifiedIp );
    if ( useSpecifiedIp )
    {
      localIp_ = specifiedIp;
    }
  }
  return localIp_;
}



// Return the session ID, generating one if it doesn't exist
const QString& Application::getSessionId()
{
  if ( sessionId_.isEmpty() )
  {
    sessionId_ = generateSessionId();
  }
  return sessionId_;
}



// A command for the application was received (i.e. "Accept" or "Reject")
void Application::gotCommand(QString command)
{
  if ( command == "accept" )
  {
    contactStarted2_UserAccepts();
  }
  else if ( command == "cancel" )
  {
    userCancelled();
  }
  else if ( command == "reject" )
  {
    userRejected();
  }

}



// A message for the application was received
void Application::gotMessage(const MimeMessage& message)
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - An application message was received" << endl;
#endif
  // Check if this is invite, accept, or cancel
  if ( message.getValue("Invitation-Command") == "INVITE" )
  {
    // This is the start of a contact-started app.  Get the app's cookie
    cookie_ = message.getValue("Invitation-Cookie");
    // Call the first step of the process
    userStartedApp_ = false;
    contactStarted1_ContactInvitesUser(message);
  }
  else if ( message.getValue("Invitation-Command") == "ACCEPT" )
  {
    // Call the next step of the process, depending on who started the app
    if ( userStartedApp_ )
    {
      userStarted2_ContactAccepts( message );
    }
    else
    {
      contactStarted3_ContactConfirmsAccept( message );
    }
  }
  else if ( message.getValue("Invitation-Command") == "CANCEL" )
  {
    // The contact canceled the app.
    if ( message.getValue("Cancel-Code") == "CANCEL" )
    {
      contactCancelled();
    }
    else if ( message.getValue("Cancel-Code") == "REJECT" )
    {
      contactRejected();
    }
  }
}



// Let the user accept or reject the application
void Application::offerAcceptOrReject(const QString& appHtml)
{
  QString acceptUrl, html, rejectUrl;

  // Create the urls for accept and reject.
  acceptUrl  = "<a href=\"kmess://accept?"  + getCookie() + "\">" + i18n("accept") + "</a>";
  rejectUrl  = "<a href=\"kmess://reject?"  + getCookie() + "\">" + i18n("cancel") + "</a>";
  // Create the message, starting with the application's particulars
  html += appHtml;
/*
  html += "<font color=purple>";
  html += "  " + i18n("Do you want to") + " ";
  html += acceptUrl;
  html += " " + i18n("or") + " ";
  html += rejectUrl;
  html += ".</font>";
*/
  html += "<font color=purple>";
  html += i18n( "Do you want to <a href=%1>accept</a> or <a href=%2>cancel</a>?" ).arg( "\"kmess://accept?"  + getCookie() + "\"" ).arg( "\"kmess://reject?"  + getCookie() + "\"" );
  html += "</font>";
  // Send the message to the chat window.
  emit messageToUser( html );
}



// Let the user cancel the application
void Application::offerCancel(const QString& appHtml)
{
  QString cancelUrl, html;

  // Create the urls for accept and reject.
  cancelUrl  = "<a href=\"kmess://cancel?"  + getCookie() + "\">" + i18n("cancel") + "</a>";
  // Create the message, starting with the application's particulars
  html += appHtml;
  html += "<font color=purple>  " + i18n("Click to <a href=%1>cancel</a>").arg( "\"kmess://cancel?"  + getCookie() + "\"" ) + ".</font>";

  // Send the message to the chat window.
  emit messageToUser( html );
}



// Send a cancel message
void Application::sendCancelMessage(bool reject)
{
  MimeMessage message;

  message.addField( "Invitation-Command", "CANCEL"    );
  message.addField( "Invitation-Cookie",  getCookie() );
  if ( reject )
  {
    message.addField( "Cancel-Code", "REJECT" );
  }
  else
  {
    message.addField( "Cancel-Code", "CANCEL" );
  }
  sendMessage( message );
}



// Send a message to the contact
void Application::sendMessage(const MimeMessage& message)
{
  MimeMessage mainMessage;

  mainMessage.addField("MIME-Version", "1.0");
  mainMessage.addField("Content-Type", "text/x-msmsgsinvite; charset=UTF-8");
  mainMessage.setBody( message.getFields() );

  emit putMsg( mainMessage.getMessage() );
}



// Start the application
void Application::start()
{
  userStartedApp_ = true;
  cookie_ = generateCookie();

  // Start the application
  userStarted1_UserInvitesContact();
}



// The user cancelled the application
void Application::userCancelled()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - user cancelled." << endl;
#endif
  QString html;

  // Create a cancel message for the user.
  html  = "<font color=purple>" + i18n("You have cancelled the invitation.") + "</font>";
  emit messageToUser( html );

  // Send a "cancel" message to the contact
  sendCancelMessage();

  // Signal that this receive transfer should be deleted.
  emit deleteMe( getCookie() );
}



// The user rejected the application
void Application::userRejected()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - user rejected." << endl;
#endif
  QString html;

  // Create a cancel message for the user.
  html  = "<font color=purple>" + i18n("You have rejected the invitation.") + "</font>";
  emit messageToUser( html );

  // Send a "cancel" message to the contact
  sendCancelMessage(true);

  // Signal that this receive transfer should be deleted.
  emit deleteMe( getCookie() );
}



// Step one of a user-started chat: the user invites the contact
void Application::userStarted1_UserInvitesContact()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - userStarted1_UserInvitesContact" << endl;
#endif
}



// Step two of a user-started chat: the contact accepts
void Application::userStarted2_ContactAccepts(const MimeMessage& /*message*/)
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - userStarted2_ContactAccepts" << endl;
#endif
}



// Step three of a user-started chat: the user confirms the accept
void Application::userStarted3_UserConfirmsAccept()
{
#ifdef KMESSDEBUG_APPLICATION
  kdDebug() << "Application - userStarted3_UserConfirmsAccept" << endl;
#endif
}
