/***************************************************************************
                          application.h  -  description
                             -------------------
    begin                : Thu Mar 20 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef APPLICATION_H
#define APPLICATION_H

#include <qobject.h>
#include <qstringlist.h>

// Forward declarations
class MimeMessage;

/**The base class of chat applications, such as file transfer and meetings.
  *@author Mike K. Bennett
  */

class Application : public QObject
{
  Q_OBJECT

  public:
    // The constructor
  	                       Application(const QString& localIp);
    // The destructor
  	virtual               ~Application();
    // Return the application's identifying cookie
    const QString&         getCookie() const;
    // A command for the application was received (i.e. "Accept" or "Reject")
    void                   gotCommand(QString command);
    // A message for the application was received
    virtual void           gotMessage(const MimeMessage& message);
    // Start the application
    void                   start();

  protected: // Protected methods
    // The contact cancelled the application
    virtual void           contactCancelled();
    // The contact rejected the application
    virtual void           contactRejected();
    // Step one of a contact-started chat: the contact invites the user
    virtual void           contactStarted1_ContactInvitesUser(const MimeMessage& message);
    // Step two of a contact-started chat: the user accepts
    virtual void           contactStarted2_UserAccepts();
    // Step three of a contact-started chat: the contact confirms the accept
    virtual void           contactStarted3_ContactConfirmsAccept(const MimeMessage& message);
    // Free a used port
    static void            freePort(const int& port);
    // Return an available port
    static int             getAvailablePort();
    // Return the local IP address, confirming it with the user if neccessary
    const QString&         getLocalIp(bool confirm = false);
    // Return the session ID, generating one if it doesn't exist
    const QString&         getSessionId();
    // Let the user accept or reject the application
    void                   offerAcceptOrReject(const QString& appHtml);
    // Let the user cancel the application
    void                   offerCancel(const QString& appHtml);
    // Send a cancel message
    void                   sendCancelMessage(bool reject = false);
    // Send a message to the contact
    void                   sendMessage(const MimeMessage& message);
    // The user cancelled the application
    virtual void           userCancelled();
    // The user rejected the application
    virtual void           userRejected();
    // Step one of a user-started chat: the user invites the contact
    virtual void           userStarted1_UserInvitesContact();
    // Step two of a user-started chat: the contact accepts
    virtual void           userStarted2_ContactAccepts(const MimeMessage& message);
    // Step three of a user-started chat: the user confirms the accept
    virtual void           userStarted3_UserConfirmsAccept();

  protected: // Protected attributes

  private: // Private methods
    // Generate a random file or authorization cookie.
    QString                generateCookie() const;
    // Generate a random session ID
    QString                generateSessionId() const;

  private: // Private attributes
    // The applications identifying cookie
    QString                cookie_;
    // The local IP address
    QString                localIp_;
    // A session ID
    QString                sessionId_;
    // A list of used ports
//    static QStringList     usedPorts_;
//    static int             usedPorts_[10];
    // Whether or not the user started the app
    bool                   userStartedApp_;

  signals:

  signals: // Signals
    // Signal that this application's activity has ended and that it should be deleted.
    void                   deleteMe(QString cookie);
    // Give a formatted message to be displayed to the user in the chat window.
    void                   messageToUser(QString html);
    // Give the switchboard a message to send to the server.
    void                   putMsg(QString message);
};

#endif
