/***************************************************************************
                          ipdialog.cpp  -  description
                             -------------------
    begin                : Wed Oct 16 2002
    copyright            : (C) 2002 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ipdialog.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qsize.h>

#include <kdebug.h>
#include <klocale.h>
#include <krestrictedline.h>
#include <krun.h>
#include <kurl.h>
#include <kurllabel.h>

IpDialog::IpDialog(QWidget *parent, const char *name )
: KDialogBase( parent, name, false, i18n( "Confirm IP Address" ), Ok | Cancel, Ok, true )
{
  // Set the dialog size
  setInitialSize( configDialogSize("IpDialog") );

  // Create the widgets
  QWidget *page = new QWidget( this );
  setMainWidget( page );
  QVBoxLayout *mainLayout = new QVBoxLayout( page, 0, spacingHint() );

  QHBoxLayout *topLayout = new QHBoxLayout( 0, 0, 0, "labelLayout" );

  QLabel *infoLabel = new QLabel( i18n( "Use this IP address:" ), page, "info" );
  topLayout->addWidget( infoLabel );

  ipLabel_ = new QLabel( page, "info" );
  topLayout->addWidget( ipLabel_ );

  mainLayout->addLayout( topLayout );   

  QLabel *infoLabel2 = new QLabel( i18n( "or enter an IP to use:" ), page, "info" );
  mainLayout->addWidget( infoLabel2 );

  ipEdit_ = new KRestrictedLine( page, "handleedit" );
  ipEdit_->setMinimumWidth( fontMetrics().maxWidth()*20 );
  // Restrict the IP address input string to numbers and periods
  ipEdit_->setValidChars("0123456789.");

  mainLayout->addWidget( ipEdit_ );

  KURLLabel *urlLabel = new KURLLabel( "www.whatismyip.com", i18n( "Open www.whatismyip.com" ), page, "info" );
  mainLayout->addWidget( urlLabel );
  connect( urlLabel, SIGNAL( leftClickedURL() ), this, SLOT( openIpUrl() ) );

  mainLayout->addStretch( 10 );

  adjustSize();

}



IpDialog::~IpDialog()
{
}



// Get an IP address from the user, or null if a new IP wasn't entered. 
bool IpDialog::getIpAddress(QString &ipAddress)
{
  // Show the detected IP in the label
  ipLabel_->setText( ipAddress );
  ok_ = false;
  // Show the window modally
  exec();
  // If ok,
  if ( ok_ )
  {
    // If the ip address is not null,
    if ( ipEdit_->text() != "" )
    {
      // Set it to the given IP
      ipAddress = ipEdit_->text();
    }
  }

  // Return whether or not OK was pressed
  return ok_;
}


// Open "www.whatismyip.com"
void IpDialog::openIpUrl()
{
  KURL   ipUrl( "http://www.whatismyip.com" );

  new KRun( ipUrl );
}



// The Cancel button was pressed.
void IpDialog::slotCancel()
{
  saveDialogSize("IpDialog");
  reject();
}



// The OK button was pressed.
void IpDialog::slotOk()
{
  saveDialogSize("IpDialog");
  ok_ = true;
  accept();
}
