/***************************************************************************
                          receivefile.cpp  -  description
                             -------------------
    begin                : Sat Mar 22 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "receivefile.h"

#include <qdir.h>

#include <kdebug.h>
#include <kextsock.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kstringhandler.h>

#include "../../kmessdebug.h"
#include "../mimemessage.h"

// The constructor
ReceiveFile::ReceiveFile(const QString& handle, const QString& localIp)
 : TransferFile(handle, localIp)
{
}



// The destructor
ReceiveFile::~ReceiveFile()
{
}



// Cancel the transfer
void ReceiveFile::cancelTransfer()
{
  // Send a cancel code?
#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveTransfer - Cancel the transfer." << endl;
#endif

  // Fail the transfer
  mode_ = CANCEL;
}



// Open the socket to the given IP address and port.  Return whether the connection was opened successfully.
bool ReceiveFile::connectToIp(QString ip, int port)
{
  bool success = false;

#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - ConnectToIp: The port is " << port << ".  Create the socket." << endl;
#endif
  // Create the socket
  socket_ = new KExtendedSocket( ip, port, 0x00 | 0x600000 );

  if ( socket_ != 0 )
  {
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Socket is non null" << endl;
#endif
    socket_->enableRead(true);
    socket_->enableWrite(true);
    socket_->setTimeout(10);

#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Connect to the server" << endl;
#endif
    // Connect to the server
    if ( socket_->connect() == 0 )
    {
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - The connect succeeded.  Connect socket signals to slots." << endl;
#endif
      // Connect the socket to receive data and report errors
  	  connect( socket_, SIGNAL( readyRead()           ),
               this,    SLOT  ( dataReceived()        ) );
  	  connect( socket_, SIGNAL( readyWrite()          ),
               this,    SLOT  ( writeCommand()        ) );
    	connect( socket_, SIGNAL( connectionFailed(int) ),
               this,    SLOT  ( socketError(int)      ) );
      // Set that the connection was successfully created
      success = true;
      // Show socket flags
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - Socket flags:  " << socket_->socketFlags() << endl;
      kdDebug() << "ReceiveFile - Socket status: " << socket_->socketStatus() << endl;
#endif
    }
/* Returning false will cause the transfer to be failed... This causes crashiness
    else
    {
      failTransfer( i18n("failed.  A connection could not be made") );
    }
*/
  }
  else
  {
    kdDebug() << "ReceiveFile - Couldn't create new socket" << endl;
  }
  // Return the connection success.
  return success;
}



// Step one of a contact-started chat: the contact invites the user
void ReceiveFile::contactStarted1_ContactInvitesUser(const MimeMessage& message)
{
#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - contactStarted1_ContactInvitesUser" << endl;
#endif
  QString html, fileSizeString;

  // Get the file name and size from the message
  fileName_      = message.getValue("Application-File");
  fileSizeString = message.getValue("Application-FileSize");
  fileSize_ = fileSizeString.toInt();

  // Create a confirmation message for the user
/*
  html  = "<font color=purple>";
  html += i18n("Do you want to accept the file: ");
  html += "</font><font color=red>";
  html += fileName_;
  html += "</font><font color=purple>";
  html += " ( " + fileSizeString + " bytes )";
  html += "</font>";
*/
  html  = "<font color=purple>";
  html += i18n("Do you want to accept the file: %1 (%2 bytes)").arg( "<font color=red>" + fileName_ + "</font>" ).arg( fileSizeString );
  html += "</font>";

  // Send the message to the chat window.
  offerAcceptOrReject( html );
}



// Step two of a contact-started chat: the user accepts
void ReceiveFile::contactStarted2_UserAccepts()
{
#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - contactStarted2_UserAccepts" << endl;
#endif
  MimeMessage message;
  // Create the accept message
  message.addField( "Invitation-Command", "ACCEPT"         );
  message.addField( "Invitation-Cookie",  getCookie()      );
  message.addField( "Launch-Application", "FALSE"          );
  message.addField( "Request-Data",       "IP-Address"     );

  sendMessage( message );
}



// Step three of a contact-started chat: the contact confirms the accept
void ReceiveFile::contactStarted3_ContactConfirmsAccept(const MimeMessage& message)
{
#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - contactStarted3_ContactConfirmsAccept" << endl;
#endif
  QString ip;
  int     port;
  // Pull the IP, port, and authorization info from the message
  ip          = message.getValue( "IP-Address" );
  port        = message.getValue( "Port"       ).toInt();
  authCookie_ = message.getValue( "AuthCookie" );

#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - IP: " << ip << " Port: " << port << " AuthCookie: " << authCookie_ << endl;
#endif
  // Start the transfer
  startTransfer( ip, port );
}



// This is called when data is received from the socket.
void ReceiveFile::dataReceived()
{
//  bool alreadyReadHead = false;
  int  bytesAvailable, bytesRead, code, size1, size2, blockSize;
  char buffer[2049], codeChar;
  bool enoughData = true, fullBlock, lessThanBlockRemaining;

  // Check what mode the transfer is in.  This will tell whether to examine the first byte for a
  if ( mode_ == RECEIVE )
  {
    // Loop while there is enough data in the buffer.
    while ( enoughData )
    {
      bytesAvailable = socket_->bytesAvailable();
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - Data received.  " << bytesAvailable << " bytes available." << endl;
#endif
      // There is enough data to read if the number of bytes available
      //  is greater than 2048 or the amount of data remaining to
      //  be received, whichever is lower
      fullBlock = ( bytesAvailable > 2048 );
      lessThanBlockRemaining = ( bytesAvailable >= ( bytesRemaining_ + 3 ) );
      enoughData = fullBlock || lessThanBlockRemaining;

      if ( bytesRemaining_ < 0 )
      {
        kdDebug() << "ReceiveFile - WARNING - Bytes remaining = " << bytesRemaining_ << "!" << endl;
        mode_ = CLOSE;
        return;
      }

      if ( !enoughData )
      {
#ifdef KMESSDEBUG_RECEIVEFILE
        kdDebug() << "ReceiveFile - Only " << bytesAvailable << " bytes available.  Wait.  "
                  << bytesRemaining_ << " bytes remaining" << endl;
#endif
        // Exit until enough data is available.
      }
      else
      {
#ifdef KMESSDEBUG_RECEIVEFILE
        kdDebug() << "ReceiveFile - " << bytesRemaining_ << " bytes in the file left to download." << endl;
        kdDebug() << "ReceiveFile - Enough Data... Proceed...." << endl;
#endif
        // The first byte in the stream indicates whether this block is data, 0, or control, 1.
        codeChar = socket_->getch();
        code = charToInt( codeChar );

#ifdef KMESSDEBUG_RECEIVEFILE
        kdDebug() << "ReceiveFile - Code is " << code << ", code char is " << codeChar << "." << endl;
#endif
#ifdef KMESSTEST
        ASSERT( ( code == 0 ) || ( code == 1 ) );
#endif

        if ( code == 0 )
        {
#ifdef KMESSDEBUG_RECEIVEFILE
          kdDebug() << "ReceiveFile - This is a data block." << endl;
#endif
          // This is a file data block.
          // The next two bytes in the control section indicate the file size.
          size1 = charToInt( socket_->getch() );
          size2 = charToInt( socket_->getch() );

#ifdef KMESSDEBUG_RECEIVEFILE
          if ( ( size1 < 0 ) || ( size2 < 0 ) )
          {
            kdDebug() << "ReceiveFile - Size controls returned negative - no data to read." << endl;
          }
#endif
          // Calculate the amount of data in the block.  This should be 2045 for most blocks, less for the final block.
          blockSize = size1 + 256 * size2;
#ifdef KMESSDEBUG_RECEIVEFILE
          kdDebug() << "ReceiveFile - Block size is " << blockSize << " = " << size1 << " + 256 * " << size2 << endl;
#endif
          // Check the blockSize - ensure that it's not too big or too small.
          if ( blockSize > 2045 )
          {
            kdDebug() << "WARNING: Unusually large block size of " << blockSize << ".  Resetting to 2045" << endl;
            blockSize = 2045;
          }
          else if ( blockSize < 0 )
          {
            kdDebug() << "WARNING: Negative block size of " << blockSize << ".  Resetting to amount of data remaining." << endl;
            blockSize = bytesRemaining_;
          }
          // Read in the block from the socket.
          bytesRead = socket_->readBlock( buffer, blockSize );
#ifdef KMESSDEBUG_RECEIVEFILE
          kdDebug() << "ReceiveFile - Read in " << bytesRead << " bytes." << endl;
#endif
          // Check the number of bytes read in.
          if ( bytesRead < 0 )
          { // An error condition was returned
            kdDebug() << "WARNING: Block not read.  No data?" << endl;
            // Set the amount of data remaining to zero in order to exit.
            bytesRemaining_ = 0;
          }
          else if ( bytesRead != blockSize )
          { // More or fewer blocks than specified were read.
            kdDebug() << "WARNING: Wanted " << blockSize << " bytes, got " << bytesRead << " bytes." << endl;
            bytesRemaining_ -= bytesRead;
          }
          else
          {
            bytesRemaining_ -= bytesRead;
          }
          // Update the progress dialog
          updateProgressDialog( fileSize_ - bytesRemaining_ );
          // Write the data read in to the file.
          file_->writeBlock( buffer, bytesRead );
          // Check whether the file is completed.
          if ( bytesRemaining_ == 0 )
          { // We've finished
#ifdef KMESSDEBUG_RECEIVEFILE
            kdDebug() << "ReceiveFile - File transfer finished" << endl;
#endif
            mode_ = CLOSE;
            return;
          }
        }
        else if ( code == 1 )
        {
          // This is a control block.
#ifdef KMESSDEBUG_RECEIVEFILE
          kdDebug() << "ReceiveFile - Control block received";
#endif
          char c1 = socket_->getch();
          char c2 = socket_->getch();
          if ( ( c1 == 0 ) && ( c2 == 0 ) )
          {
            // The transfer was cancelled by the sender.
            failTransfer( i18n("was cancelled by the sender") );
          }
        }
      }
    }
  }
  else
  { // The mode is not receive, therefore check for the initial control stuff.
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Check for control data" << endl;
#endif
    bytesAvailable = socket_->bytesAvailable();
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Data received.  " << bytesAvailable << " bytes available." << endl;
#endif
    if ( bytesAvailable >= 2048 )
    {
      bytesAvailable = 2048;
    }
    // Must be control data, so send back that byte
    bytesRead = socket_->readBlock( buffer, bytesAvailable );
    buffer[ bytesRead + 1 ] = '\0'; // Make it a proper string
    QString data = QString::fromUtf8( buffer );
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Control Data : " << data << endl;
#endif
    if (data.left(10) == "VER MSNFTP")
    {
      mode_ = SENDUSR;
    }
    else if (data.left(3) == "FIL")
    {
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - newline found at " << data.find("\r\n") << "." << endl;
#endif
      // Strip data from anything to the right of the newline
      data = data.left( data.find("\r\n") );

      KStringHandler kstr;
      QString bytesRemaining = kstr.word( data, 1 );
      bytesRemaining_ = bytesRemaining.toInt();


#ifdef KMESSDEBUG_RECEIVEFILE
      int index = 0;
      kdDebug() << "ReceiveFile - Bytes remaining = " << bytesRemaining_ << " = " << bytesRemaining << " command = " << kstr.word( data, index ) << endl;
#endif
      mode_ = SENDTFR;
    }
    // Don't keep looking for more data.
    enoughData = false;
  }
}



// Initialize the transfer.
bool ReceiveFile::initTransfer(QString ip, int port)
{
  bool connected, success = false;

#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - Call connect to IP" << endl;
#endif
  // First connect to the ip.
  connected = connectToIp( ip, port );

  // If that worked, open the file
  if ( connected )
  {
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Attempt to open the file writeonly" << endl;
#endif
    bool open = file_->open(IO_WriteOnly);
    if ( open )
    {
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - initFileTrasnfer: Connected" << endl;
#endif
      // Ok, we are connected fine, so do a bit of handshaking
      mode_ = SENDVER;
      // Set that the connection was successfully created
      success = true;
    }
    else
    {
      kdDebug() << "initFileTrasnfer: WARNING couldn't open file" << endl;
      failTransfer( i18n("failed.  Couldn't open file") );
    }
  }
  return success;
}



// Start the transfer by connecting to the given ip:port and waiting for data.
void ReceiveFile::startTransfer(QString ip, int port)
{
  bool    connectionSuccess = false;
  QDir    homeDir;
  QString initialFilePath;

  // Set an initial path to the file
  homeDir = QDir::home();
  initialFilePath = homeDir.absPath() + "/" + fileName_;
  // Open a file dialog so that the user can save the file to a particular directory and name.
  filePath_ = KFileDialog::getSaveFileName();
#ifdef KMESSDEBUG_RECEIVEFILE
  kdDebug() << "ReceiveFile - Home dir is " << homeDir.absPath() << " Save to " << filePath_ << endl;
#endif
  if ( filePath_ != "" )
  {
    // If the user hit "OK" with some file name.
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Create new file" << endl;
#endif
    // Create the file to save to
    file_ = new QFile( filePath_ );
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - Call initTransfer to " << ip << ":" << port << endl;
#endif
    // Create the socket
    connectionSuccess = initTransfer( ip, port );
    if ( !connectionSuccess )
    {
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveFile - Connection failed" << endl;
#endif
      failTransfer( i18n("failed.  A connection could not be made") );
    }
  }
  else
  { // The user hit "cancel" on the file name.
#ifdef KMESSDEBUG_RECEIVEFILE
    kdDebug() << "ReceiveFile - User cancelled file transfer at name save" << endl;
#endif
    failTransfer( i18n("was cancelled") );
  }
}



// The socket is ready for writing.  Write any outstanding commands.
void ReceiveFile::writeCommand()
{
  switch ( mode_ )
  {
    case WAITING :
    {
      // Do nothing.  Wait.
      break;
    }
    case SENDVER :
    {
      // This is the version confirmation.  Send our version.
      writeMessage("VER MSNFTP\r\n\0");
      // Wait for USR
      mode_ = WAITING;
      break;
    }
    case SENDUSR :
    {
      // This is the user confirmation.  Send the file size.
      QString message;
      writeMessage("USR " + getHandle() + " " + authCookie_ + "\r\n");
      // Wait for TFR
      mode_ = WAITING;
      break;
    }
    case SENDTFR :
    {
      // This is the user confirmation.  Sent TFR to start the download.
      writeMessage("TFR\r\n");
      // Initialize the progress dialog
      QString text = i18n("Receiving file ") + fileName_;
      initializeProgressDialog( text );
      // Wait for the file to be sent.
      mode_ = RECEIVE;
      break;
    }
    case RECEIVE :
    {
      // Do nothing.  Wait.
      break;
    }
    case CANCEL :
    {
#ifdef KMESSDEBUG_RECEIVEFILE
      kdDebug() << "ReceiveTransfer - CANCEL TRANSFER AT RECEIVE" << endl;
#endif
      char out[3];
      out[0] = intToChar( 1 );
      out[1] = intToChar( 0 );
      out[2] = intToChar( 0 );
      socket_->writeBlock( out, 3 );

      mode_ = WAITING;
      failTransfer( i18n("was cancelled") );
      break;
    }
    case CLOSE :
    {
      // Send the bye
      QString message;
      writeMessage("BYE 16777989\r\n");
      // The file transfer finished successfully
      successfulTransfer();
      // Wait for the file to be sent.
      mode_ = WAITING;
      break;
    }
  }
}
