/***************************************************************************
                          receivefile.h  -  description
                             -------------------
    begin                : Sat Mar 22 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RECEIVEFILE_H
#define RECEIVEFILE_H

#include <qwidget.h>
#include "transferfile.h"

/**The class for receiving a file from a contact.
  *@author Mike K. Bennett
  */

class ReceiveFile : public TransferFile
{
  Q_OBJECT

  public:
    // The constructor
                           ReceiveFile(const QString& handle, const QString& localIp);
    // The destructor
  	                      ~ReceiveFile();

  private: // Private methods
    // Open the socket to the given IP address and port.  Return whether the connection was opened successfully.
    bool                   connectToIp(QString ip, int port);
    // Step one of a contact-started chat: the contact invites the user
    void                   contactStarted1_ContactInvitesUser(const MimeMessage& message);
    // Step two of a contact-started chat: the user accepts
    void                   contactStarted2_UserAccepts();
    // Step three of a contact-started chat: the contact confirms the accept
    void                   contactStarted3_ContactConfirmsAccept(const MimeMessage& message);
    // Initialize the transfer.
    bool                   initTransfer(QString ip, int port);
    // Start the transfer by connecting to the given ip:port and waiting for data.
    void                   startTransfer(QString ip, int port);

  private slots: // Protected slots
    // Cancel the transfer
    void                   cancelTransfer();
    // This is called when data is received from the socket.
    void                   dataReceived();
    // The socket is ready for writing.  Write any outstanding commands.
    void                   writeCommand();

  private: // Private attributes
    // The authorization cookie for the transfer
    QString                authCookie_;
    // The mode of the transfer.
    enum Mode            { WAITING = 0, SENDVER = 1, SENDUSR = 2,
                           SENDTFR = 3, RECEIVE = 4, CANCEL = 5, CLOSE = 6
                         } mode_;

};

#endif
