/***************************************************************************
                          sendfile.cpp  -  description
                             -------------------
    begin                : Mon Mar 24 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sendfile.h"

#include <qdir.h>
#include <qprocess.h>
#include <qregexp.h>

#include <kdebug.h>
#include <kextsock.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kstringhandler.h>

#include "../../kmessdebug.h"
#include "../mimemessage.h"

// The constructor
SendFile::SendFile(const QString& handle, const QString& localIp)
 : TransferFile(handle, localIp),
   localPort_(-1),
   server_(NULL)
{
}



// The destructor
SendFile::~SendFile()
{
}



// Accept incoming connections on the socket.
void SendFile::acceptConnection()
{
  int acceptResult;

#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Accept a socket" << endl;
#endif
  acceptResult = server_->accept(socket_);

  if ( acceptResult == 0 )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Accept success" << endl;
#endif
    newConnection(socket_);
  }
#ifdef KMESSDEBUG_SENDFILE
  else if ( acceptResult == -1 )
  {
    kdDebug() << "SendFile - Accept socket error" << endl;
  }
  else if ( acceptResult == -2 )
  {
    kdDebug() << "SendFile - Accept error - socket is not passive" << endl;
  }
  else if ( acceptResult == -3 )
  {
    kdDebug() << "SendFile - Accept error - timeout" << endl;
  }
#endif
}



// Cancel the transfer
void SendFile::cancelTransfer()
{
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendTransfer - Cancel the transfer." << endl;
#endif
  // Send a cancel code
  mode_ = CANCEL;
}



// This is called when data is received from the socket.
void SendFile::dataReceived()
{
  int      blockSize, bytesAvailable, bytesRead;
  char     buffer[2050];
  QString  data;

#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Data received" << endl;
#endif

  bytesAvailable = socket_->bytesAvailable();
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - " << bytesAvailable << " bytes available.";
#endif
  if ( bytesAvailable > 2048 )
  {
    blockSize = 2048;
  }
  else if ( bytesAvailable < 0 )
  {
    blockSize = 0;
  }
  else
  {
    blockSize = bytesAvailable;
  }
  // Read in the block from the socket.
  bytesRead = socket_->readBlock( buffer, blockSize );
  buffer[2048] = '\0'; // Make sure it's null terminated.
  buffer[2049] = 0; // Make doubly sure since the sprintf line seems to fail.
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Read in " << bytesRead << " bytes." << endl;
#endif
  data.sprintf("%s", buffer);
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Printed buffer to data - analyzing." << endl;
#endif
  if ( data.left(10) == "VER MSNFTP" )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Send VER" << endl;
#endif
    // Switch mode to sendver
    mode_ = SENDVER;
  }
  else if ( data.left(3) == "USR" )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Send FIL" << endl;
#endif
    // Switch mode to SENDFIL
    mode_ = SENDFIL;
  }
  else if ( data.left(3) == "TFR" )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Send the file" << endl;
#endif
    // Initialize the progress dialog
    QString text = i18n("Sending file %1").arg( fileName_ );
    initializeProgressDialog( text );
    // Send the file.
    mode_ = SEND;
  }
  else if ( data.left(3) == "BYE" )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Got Bye" << endl;
    kdDebug() << "SendFile - Close socket - Transfer is done." << endl;
#endif
    // Close
    successfulTransfer();
  }
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Message parsed" << endl;
#endif
}



// Disconnect the socket.  Close the file.
void SendFile::disconnect()
{
  // Call the parent's disconnect function
  TransferFile::disconnect();
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - Close the server" << endl;
#endif
  // Delete the server.
  if ( server_ != 0 )
  {
    server_->closeNow();
    delete server_;
    server_ = 0;
  }
}



// Return the local port, getting an available one if neccessary
int SendFile::getLocalPort()
{
  if ( localPort_ < 0 )
  {
    localPort_ = getAvailablePort();
  }
  return localPort_;
}



// Open a connection and listen on the local port for a transfer.
void SendFile::listen()
{
  unsigned short int port;

#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "ATTEMPTING TO OPEN CONNECTION FOR LISTEN" << endl;
#endif
  // Convert the port to an unsigned int
  port = getLocalPort();
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - SendFile::listen() - Port is " << port << endl;
#endif
  // Create the server socket to the given port.
  server_ = new KExtendedSocket();

  // Set the socket port
  if ( server_ != 0 )
  {
    // Connect the server to signal when it is ready to accept a connection
    connect( server_, SIGNAL( readyAccept()        ),
             this,    SLOT  ( acceptConnection() ) );
    if ( server_->setPort(port) )
    {
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - Port was set to " << port << endl;
#endif
    }
#ifdef KMESSDEBUG_SENDFILE
    else
    {
      kdDebug() << "SendFile - Port wasn't set." << endl;
    }
#endif
    server_->setSocketFlags(  KExtendedSocket::noResolve
                            | KExtendedSocket::passiveSocket
                            | KExtendedSocket::anySocket     );
    // Put the socket in listen mode.
    int listenResult = server_->listen(1);

#ifdef KMESSDEBUG_SENDFILE
    if ( listenResult == 0 )
    {
      kdDebug() << "SendFile - Listen OK on port " << port << endl;
    }
    else if ( listenResult == -1 )
    {
      kdDebug() << "SendFile - Listen socket error" << endl;
    }
    else if ( listenResult == -2 )
    {
      kdDebug() << "SendFile - Listen error - socket is not passive" << endl;
    }
#endif
    server_->setBlockingMode(true);
  }
  else
  {
    // Creating the socket failed
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - ouldn't create socket" << endl;
#endif
    failTransfer( i18n("failed.  Couldn't create a socket") );
  }
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - SOCKET IS EITHER CONNECTED OR IT ISN'T" << endl;
#endif
}



// Accept an incoming connection as a pre-connected socket.
void SendFile::newConnection(KExtendedSocket *socket)
{
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - !!!! Got new incoming socket" << endl;
#endif
  if ( socket != 0 )
  {
#ifdef KMESSDEBUG_SENDFILE
    kdDebug() << "SendFile - Socket is non-null.  Connect it up." << endl;
#endif
    // Save the socket
    socket_ = socket;

    socket_->setBlockingMode(false);
    socket_->enableRead(true);
    socket_->enableWrite(true);
    socket_->setBufferSize(-1, -1);
    // Connect the socket to receive data and report errors
    connect( socket_, SIGNAL( readyRead()           ),
             this,    SLOT  ( dataReceived()        ) );
    connect( socket_, SIGNAL( readyWrite()          ),
             this,    SLOT  ( writeData()           ) );
  	connect( socket_, SIGNAL( connectionFailed(int) ),
             this,    SLOT  ( socketError(int)      ) );


    int flagResult = socket_->setSocketFlags(   KExtendedSocket::anySocket
                                              | KExtendedSocket::inputBufferedSocket
                                              | KExtendedSocket::outputBufferedSocket );
#ifdef KMESSDEBUG_SENDFILE
    if ( flagResult != 0 )
    {
      kdDebug() << "SendFile - Setting flags failed" << endl;
    }
#endif
  }
}


// Step one of a user-started chat: the user invites the contact
void SendFile::userStarted1_UserInvitesContact()
{
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - userStarted1_UserInvitesContact" << endl;
#endif
  QString     html, sizeString;
  MimeMessage message;

  // Show a dialog to get a file path from the user.
  filePath_ = KFileDialog::getOpenFileName( );
  if ( filePath_.isEmpty() )
  {
    failTransfer( i18n("failed.  Null file path") );
    return;
  }
  // Get the file name from the path
  fileName_ = filePath_.right( filePath_.length() - filePath_.findRev( QRegExp("/") ) - 1 );
  // Create a file object
  file_ = new QFile( filePath_ );
  // Check if the file truly exists
  if ( !file_->exists() )
  {
    failTransfer( i18n("failed.  The file doesn't exist") );
    return;
  }
  // Open the file for reading
  bool open = file_->open(IO_ReadOnly);
  if ( !open )
  {
    failTransfer( i18n("failed.  Couldn't open file") );
    return;
  }
  // Get the size of the file in bytes
  fileSize_ = file_->size();
  // Store this as the amount remaining to be sent, as well.
  bytesRemaining_ = fileSize_;
  // Convert the size to a string
  sizeString.sprintf("%d", fileSize_);

  // Create the invitation message
  message.addField( "Application-Name",     "File Transfer" );
  message.addField( "Application-GUID",     getAppId()      );
  message.addField( "Invitation-Command",   "INVITE"        );
  message.addField( "Invitation-Cookie",    getCookie()     );
  message.addField( "Application-File",     fileName_       );
  message.addField( "Application-FileSize", sizeString      );

  sendMessage( message );

  // Give the user the option of cancelling the transfer
  html = "<font color=purple>";
  html += i18n("Sending file ");
  html += "</font><font color=blue>";
  html += fileName_;
  html += ".</font>";

  offerCancel( html );
}



// Step two of a user-started chat: the contact accepts
void SendFile::userStarted2_ContactAccepts(const MimeMessage& /*message*/)
{
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - userStarted2_ContactAccepts" << endl;
#endif
  QString html;

  // Create a message showing that the transfer was accepted.
  html = "<font color=blue>";
  html += i18n("Transfer accepted.");
  html += "</font>";

  // Send the message to the chat window.
  emit messageToUser( html );

  // Send the user's accept message
  userStarted3_UserConfirmsAccept();
}



// Step three of a user-started chat: the user confirms the accept
void SendFile::userStarted3_UserConfirmsAccept()
{
#ifdef KMESSDEBUG_SENDFILE
  kdDebug() << "SendFile - userStarted3_UserConfirmsAccept" << endl;
#endif
  QString     portString;
  MimeMessage message;

  portString.sprintf( "%d", getLocalPort() );
  // Create the invitation message
  message.addField( "Invitation-Command", "ACCEPT"         );
  message.addField( "Invitation-Cookie",  getCookie()      );
  message.addField( "IP-Address",         getLocalIp(true) );
  message.addField( "Port",               portString       );
  message.addField( "Launch-Application", "FALSE"          );
  message.addField( "AuthCookie",         getSessionId()   );
  message.addField( "Request-Data",       "IP-Address:"    );

  sendMessage( message );

  // Connect to the port and start listening for the transfer.
  listen();
}



// The socket is ready for writing.  Write data to it.
void SendFile::writeData()
{
  const int BLOCKSIZE = 2048;
  int       blockSize, bytesRead, bytesWritten, size1, size2;
  char      data[BLOCKSIZE - 3], out[BLOCKSIZE], c0, c1, c2, temp[10];

  switch ( mode_ )
  {
    case WAITING :
    {
      // Do nothing.  Wait.
      break;
    }
    case SENDVER :
    {
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - Send the version." << endl;
#endif
      // This is the version confirmation.  Send our version.
      writeMessage("VER MSNFTP\r\n");
      // Wait for USR
      mode_ = WAITING;
      break;
    }
    case SENDFIL :
    {
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - Send the FIL message - size = " << fileSize_ << "." << endl;
#endif
     // This is the user confirmation.  Send the file size.
      QString message;
      message.sprintf("FIL %d\r\n", fileSize_);
      writeMessage(message);
      // Wait for TFR
      mode_ = WAITING;
      break;
    }
    case SEND :
    {
      // Read a block from the file.
      bytesRead = file_->readBlock( data, BLOCKSIZE - 3 );

      if ( bytesRead > 0 )
      {
        bytesRemaining_ -= bytesRead;
      }
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - " << bytesRead << " bytes read from file.  " << bytesRemaining_ << " bytes remaining." << endl;
#endif
      // Update the progress dialog
      updateProgressDialog( fileSize_ - bytesRemaining_ );
      if ( bytesRemaining_ <= 0 )
      {
#ifdef KMESSDEBUG_SENDFILE
        kdDebug() << "SendFile - Go into waiting mode." << endl;
#endif
        // The file send *should* be done.  Go into waiting mode.
        mode_ = WAITING;
      }
      if ( bytesRead > (BLOCKSIZE - 3) )
      {
        kdDebug() << "SendFile:WARNING: Read over 2045 bytes." << endl;
        blockSize = BLOCKSIZE;
      }
      else if ( bytesRead < 0 )
      {
        kdDebug() << "SendFile:WARNING: File read error." << endl;
        blockSize = 0;
      }
      else
      {
        blockSize = bytesRead + 3;
#ifdef KMESSDEBUG_SENDFILE
        kdDebug() << "SendFile - Set the blocksize to " << blockSize << " = " << bytesRead << " + 3." << endl;
#endif
      }

      // Calculate the size controls
      size1 = (blockSize - 3)%256;//static_cast<int>(  fmod( static_cast<double>(blockSize - 3), 256.0 ) );
      size2 = (blockSize - 3)/256;//static_cast<int>( floor( static_cast<double>(blockSize - 3) / 256.0 ) );
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - " << size1 << " + 256 * " << size2 << " = " << ( size1 + 256 * size2 ) << endl;
#endif
      // Store the size values as chars.
      c1 = intToChar( size1 );
      c2 = intToChar( size2 );
      // The first control character is 0, since this is a data block.
      c0 = intToChar( 0 );
      sprintf(temp, "%c%c%c", c0, c1, c2);
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - Control chars are " << c0 << " " << c1 << " " << c2 << " = " << temp << endl;
#endif
      // Create the output block.
      out[0] = c0;
      out[1] = c1;
      out[2] = c2;
      for ( int i = 3; i < blockSize; i++ )
      {
        out[i] = data[i-3];
      }
      // Write the data to the socket_
      bytesWritten = socket_->writeBlock( out, blockSize );
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - " << bytesWritten << " bytes written to the socket (should be " << blockSize << ")" << endl;
#endif
      // Make sure that the proper number of bytes was written.
      if ( bytesWritten != blockSize )
      {
        kdDebug() << "SendFile:WARNING: " << blockSize
                  << " bytes should have been sent, " << bytesWritten
                  << " bytes actually sent." << endl;
      }
      break;
    }
    case CANCEL :
    {
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - CANCEL TRANSFER AT SEND" << endl;
#endif
      // A send transfer tells the recipient that the send is cancelled
      //  by sending 100.
      out[0] = intToChar( 1 );
      out[1] = intToChar( 0 );
      out[2] = intToChar( 0 );
      socket_->writeBlock( out, 3 );
      // Fail the transfer with a cancel
      failTransfer( i18n("was cancelled") );
      break;
    }
    case CLOSE :
    {
#ifdef KMESSDEBUG_SENDFILE
      kdDebug() << "SendFile - Close socket - Transfer is done." << endl;
#endif
      successfulTransfer();
      break;
    }
  }
}

