/***************************************************************************
                          sendfile.h  -  description
                             -------------------
    begin                : Mon Mar 24 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SENDFILE_H
#define SENDFILE_H

#include <qwidget.h>
#include "transferfile.h"

/**The class for sending a file to a contact.
  *@author Mike K. Bennett
  */

class SendFile : public TransferFile
{
  Q_OBJECT

  public:
    // The constructor
                           SendFile(const QString& handle, const QString& localIp);
    // The destructor
  	                      ~SendFile();

  private: // Private methods
    // Disconnect the socket.  Close the file.
    void                   disconnect();
    // Return the local port, getting an available one if neccessary
    int                    getLocalPort();
    // Open a connection and listen on the local port for a transfer.
    void                   listen();
    // Accept an incoming connection as a pre-connected socket.
    void                   newConnection(KExtendedSocket *socket);
    // Step one of a user-started chat: the user invites the contact
    void                   userStarted1_UserInvitesContact();
    // Step two of a user-started chat: the contact accepts
    void                   userStarted2_ContactAccepts(const MimeMessage& message);
    // Step three of a user-started chat: the user confirms the accept
    void                   userStarted3_UserConfirmsAccept();

  private slots: // Private slots
    // Accept incoming connections on the socket.
    void                   acceptConnection();
    // Cancel the transfer
    void                   cancelTransfer();
    // This is called when data is received from the socket.
    void                   dataReceived();
    // The socket is ready for writing.  Write data to it.
    void                   writeData();

  private: // Private attributes
    // The local port
    int                    localPort_;
    // The mode of the transfer.
    enum Mode            { WAITING = 0, SENDVER = 1, SENDFIL = 2, SEND = 3, CANCEL = 4, CLOSE = 5 } mode_;
    // The server socket which listens for incoming connections.
    KExtendedSocket       *server_;

};

#endif
