/***************************************************************************
                          transferfile.h  -  description
                             -------------------
    begin                : Sat Mar 22 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRANSFERFILE_H
#define TRANSFERFILE_H

#include <qwidget.h>
#include "application.h"

// Forward declarations
class KExtendedSocket;
class QFile;
class TransferProgressDialog;

/**The application providing basic stuff needed to both send and receive files.
  *@author Mike K. Bennett
  */

class TransferFile : public Application
{
  Q_OBJECT

  public:
    // The constructor
  	                       TransferFile(const QString& handle, const QString& localIp);
    // The destructor
  	                      ~TransferFile();
    // Return the application's GUID
    static QString         getAppId();

  protected: // Protected methods
    // Convert a char to an integer
    int                    charToInt(char c);
    // Disconnect the socket.  Close the file.
    virtual void           disconnect();
    // Cancel the file transfer.
    void                   failTransfer(QString text);
    // Return the user's handle
    const QString&         getHandle() const;
    // Create and initilize the progress dialog.
    void                   initializeProgressDialog(QString description);
    // Convert an integer (should be less than 256) to a char
    char                   intToChar(int i);
    // Signal the user that the tranfer was successful and delete this object.
    void                   successfulTransfer();
    // Update the progress dialog with the number of bytes transferred.
    void                   updateProgressDialog(int noBytes);
    // Write a message to the socket.
    void                   writeMessage(QString message);

  protected slots: // Protected slots
    // Cancel the transfer
    virtual void           cancelTransfer() = 0;
    // This is called when data is received from the socket.
    virtual void           dataReceived() = 0;
    // This is called when there's an error in the socket.
    void                   socketError(int error);

  protected: // Protected attributes
    // The number of bytes remaining to be transferred.
    int                    bytesRemaining_;
    // The file to be saved or sent
    QFile                 *file_;
    // The file name
    QString                fileName_;
    // The complete file path
    QString                filePath_;
    // The file size in bytes
    int                    fileSize_;
    // The socket over which data is sent or received.
    KExtendedSocket       *socket_;

  private: // Private methods
    // Delete the progress dialog.
    void                   deleteProgressDialog();

  private: // Private attributes
    // The user's handle
    QString                 handle_;
    // The file transfer progress dialog
    TransferProgressDialog *progressDialog_;

};

#endif
