/***************************************************************************
                          transferprogressdialog.cpp  -  description
                             -------------------
    begin                : Tue Jun 4 2002
    copyright            : (C) 2002 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "transferprogressdialog.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qsize.h>

#include <kdebug.h>
#include <klocale.h>
#include <kprogress.h>

#include "../../kmessdebug.h"

// The constructor
TransferProgressDialog::TransferProgressDialog(QString description, int totalBytes, QWidget *parent, const char *name )
 : KDialogBase( parent, name, false, QString::null, Cancel | Close | User1, User1, true, i18n( "&Hide" ) ),
   totalBytes_(totalBytes)
{
  // Initially disable "Close"
  enableButton( Close , false );

  // Set the dialog size
  setInitialSize( configDialogSize("TransferProgressDialog") );

  // Create the widgets
  QWidget *page = new QWidget( this );
  setMainWidget( page );

  QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );

  // Add the label that says "Downloading blah.tar.gz"
  QLabel *fileLabel = new QLabel( description, page, "info" );
  topLayout->addWidget( fileLabel );

  // Add the progress bar
  progressBar_ = new KProgress( page, "progress" );
  topLayout->addWidget( progressBar_ );

  // Add the label that says "x of y bytes transferred." 
  progressLabel_ = new QLabel( "", page, "info" );
  topLayout->addWidget( progressLabel_ );

  topLayout->addStretch( 10 );

  adjustSize();

  // Set the multiplier so that the range is divided into steps of 1000.
  multiplier_ = 1000.0/((double)totalBytes_);
  progressBar_->setTotalSteps( 1000 );
  totalBytesString_.sprintf("%d", totalBytes_);
}



// The destructor
TransferProgressDialog::~TransferProgressDialog()
{
}



// The transfer is finished.
void TransferProgressDialog::finishTransfer()
{
  QString text;

  // Disable "Cancel" and "Hide", enable "Close"
  enableButton( Close , true );
  enableButton( User1 , false );
  enableButtonCancel( false );

  // Tell the user that the transfer is complete
  text = i18n("File transfer completed.") + "  " + totalBytesString_ + i18n(" bytes transferred.");
  progressLabel_->setText( text );
}



// Update the progress bar.
void TransferProgressDialog::updateProgress(int noBytes)
{
  QString text, bytes;
  progressBar_->setProgress( (int)( (double)noBytes * multiplier_ ) );
  bytes.sprintf("%d", noBytes);
  text = "  " + bytes + i18n(" of ") + totalBytesString_ + i18n(" bytes transferred.");
  progressLabel_->setText( text );
}



// The Cancel button was pressed.
void TransferProgressDialog::slotCancel()
{
#ifdef KMESSDEBUG_TRANSFERPROGRESS
  kdDebug() << "TransferProgressDialog - Cancelling" << endl;
#endif
  saveDialogSize("TransferProgressDialog");
  emit cancelTransfer();
}



// The Close button was pressed.
void TransferProgressDialog::slotClose()
{
#ifdef KMESSDEBUG_TRANSFERPROGRESS
  kdDebug() << "TransferProgressDialog - Closing" << endl;
#endif
  saveDialogSize("TransferProgressDialog");
  reject();
}



// The Hide button was pressed.
void TransferProgressDialog::slotUser1()
{
#ifdef KMESSDEBUG_TRANSFERPROGRESS
  kdDebug() << "TransferProgressDialog - Hiding" << endl;
#endif
  hide();
}
