/***************************************************************************
                          mimemessage.cpp  -  description
                             -------------------
    begin                : Sat Mar 8 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
   The decode...() functions and getCodecByName() come from KMail
   and are therefore (C) KMail developers.
*/

#include "mimemessage.h"

#include <qregexp.h>
#include <qtextcodec.h>

#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kcharsets.h>

#include <mimelib/mimepp.h>

#include "../kmessdebug.h"

//#define KMESSDEBUG_MIMEMESSAGE


// The constructor
MimeMessage::MimeMessage()
 : QObject()
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Constructor." << endl;
#endif
}



// The constructor that parses a message
MimeMessage::MimeMessage(QString message)
 : QObject()
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Constructor." << endl;
#endif
  parseMessage( message );
}



// The copy constructor
MimeMessage::MimeMessage(const MimeMessage& other)
 : QObject()
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Copy constructor." << endl;
#endif
  QString field, value;

  // Get the fields and values from the other message
  for ( uint i = 0; i < other.getNoFields(); i++ )
  {
    other.getFieldAndValue( field, value, i );
    fields_ << field;
    values_ << value;
  }
  // Get the body of the other message
  other.getBody( body_ );
}



// The destructor
MimeMessage::~MimeMessage()
{
}



// Add a field to the message
void MimeMessage::addField(const QString& field, const QString& value)
{
  fields_ << field;
  values_ << value;
}


// decodes MIME strings like =?iso...=...?= ...
QString MimeMessage::decodeRFC2047String(const QCString& aStr) const
{
  QString result;
  QCString charset;
  char *pos, *beg, *end, *mid=0;
  QCString str, cstr, LWSP_buffer;
  char encoding='Q', ch;
  bool valid, lastWasEncodedWord=FALSE;
  const int maxLen=200;
  int i;

  if (aStr.find("=?") < 0)
  {
    //QString str = messageCodec->toUnicode(aStr);
    QString str = QString::fromUtf8( aStr );
    if (str.find('\n') == -1) return str;
    QString str2((QChar*)0, str.length());
    uint i = 0;
    while (i < str.length())
    {
      if (str[i] == '\n')
      {
        str2 += ' ';
        i += 2;
      } else {
        str2 += str[i];
        i++;
      }
    }
    return str2;
  }

  for (pos=aStr.data(); *pos; pos++)
  {
    // line unfolding
    if ( pos[0] == '\r' && pos[1] == '\n' ) {
      pos++;
      continue;
    }
    if ( pos[0] == '\n' )
      continue;
    // collect LWSP after encoded-words,
    // because we might need to throw it out
    // (when the next word is an encoded-word)
    if ( lastWasEncodedWord && ( pos[0] == ' ' || pos[0] == '\t' ) ) {
      LWSP_buffer += pos[0];
      continue;
    }
    // verbatimly copy normal text
    if (pos[0]!='=' || pos[1]!='?')
    {
      result += LWSP_buffer + pos[0];
      LWSP_buffer = 0;
      lastWasEncodedWord = FALSE;
      continue;
    }
    // found possible encoded-word
    beg = pos+2;
    end = beg;
    valid = TRUE;
    // parse charset name
    charset = "";
    for (i=2,pos+=2; i<maxLen && (*pos!='?'&&(*pos==' '||ispunct(*pos)||isalnum(*pos))); i++)
    {
      charset += *pos;
      pos++;
    }
    if (*pos!='?' || i<4 || i>=maxLen) valid = FALSE;
    else
    {
      // get encoding and check delimiting question marks
      encoding = toupper(pos[1]);
      if (pos[2]!='?' || (encoding!='Q' && encoding!='B'))
	valid = FALSE;
      pos+=3;
      i+=3;
    }
    if (valid)
    {
      mid = pos;
      // search for end of encoded part
      while (i<maxLen && *pos && !(*pos=='?' && *(pos+1)=='='))
      {
	i++;
	pos++;
      }
      end = pos+2;//end now points to the first char after the encoded string
      if (i>=maxLen || !*pos) valid = FALSE;
    }
    if (valid)
    {
      // valid encoding: decode and throw away separating LWSP
      ch = *pos;
      *pos = '\0';
      str = QCString(mid).left((int)(mid - pos - 1));
      if (encoding == 'Q')
      {
	// decode quoted printable text
	for (i=str.length()-1; i>=0; i--)
	  if (str[i]=='_') str[i]=' ';
	cstr = decodeQuotedPrintable(str);
      }
      else
      {
	// decode base64 text
	cstr = decodeBase64(str);
      }
      QTextCodec *codec = getCodecByName(charset);
      if (!codec)
      {
        result += QString::fromUtf8( cstr );
      }
      else
      {
        result += codec->toUnicode(cstr);
      }

      lastWasEncodedWord = TRUE;

      *pos = ch;
      pos = end -1;
    }
    else
    {
      // invalid encoding, keep separating LWSP.
      //result += "=?";
      //pos = beg -1; // because pos gets increased shortly afterwards
      pos = beg - 2;
      result += LWSP_buffer;
      result += *pos++;
      result += *pos;
      lastWasEncodedWord = FALSE;
    }
    LWSP_buffer = 0;
  }
  return result;
}



// decodes a quoted-printable message into ...
QCString MimeMessage::decodeQuotedPrintable(const QCString& aStr) const
{
  QCString bStr = aStr;
  if (aStr.isNull())
    bStr = "";

  DwString dwsrc(bStr.data());
  DwString dwdest;

  DwDecodeQuotedPrintable(dwsrc, dwdest);
  return dwdest.c_str();
}


// decodes a base64 message into ...
QCString MimeMessage::decodeBase64(const QCString& aStr) const
{
  QCString bStr = aStr;
  if (aStr.isNull())
    bStr = "";
  while (bStr.length() < 16) bStr += '=';

  DwString dwsrc(bStr.data(), bStr.length());
  DwString dwdest;
  QCString result;

  DwDecodeBase64(dwsrc, dwdest);
  result = dwdest.c_str();
  return result;
}



// Return the body of the message
void MimeMessage::getBody(QString& body) const
{
  body = body_;
}



// Return the body of the message
const QString& MimeMessage::getBody() const
{
  return body_;
}



// Finds a QTextCodec by name
QTextCodec* MimeMessage::getCodecByName(const QCString& codecName ) const
{
  if ( codecName.isEmpty() )
  {
    return 0;
  }

  return KGlobal::charsets()->codecForName( codecName.lower() );
}



// Return the field and value at the given index
void MimeMessage::getFieldAndValue(QString& field, QString& value, const uint& index) const
{
  if ( index < fields_.count() )
  {
    field = fields_[index];
    value = values_[index];
  }
  else
  {
    field = QString::null;
    value = QString::null;
  }
}



// Return the message fields as a big string
QString MimeMessage::getFields() const
{
  QString message = "";

  // Get the fields and values
  for ( uint i = 0; i < fields_.count(); i++ )
  {
    message += fields_[i] + ": " + values_[i] + "\r\n";
  }

  return message;
}



// Return the entire message as a big string
QString MimeMessage::getMessage() const
{
  QString message;

  message =  getFields();
  message += "\r\n";
  // Add the body
  message += body_;

  return message;
}



// The total number of fields
uint MimeMessage::getNoFields() const
{
  return fields_.count();
}



// Get one parameter of a value that has multiple parameters
QString MimeMessage::getSubValue(const QString& field, const QString& subField) const
{
  QString value, parameter;
  int     left, right;
  // Get the value referred to by the field
  value = getValue( field );
  if ( !value.isNull() )
  {
    // If the subfield isn't specified, then get whatever is at the start of the message until the
    //  first semicolon or the end of the line
    if ( subField.isNull() )
    {
      left = 0;
    }
    else
    {
      // The left of the parameter is "subField=", the right is the next semicolon or the end of the line
      left = value.find( subField + "=" );
      if ( left >= 0 )
      {
        left += subField.length() + 1;
      }
    }
    if ( left >= 0 )
    {
      right = value.find( ";", left );
      if ( right < 0 )
      {
        right = value.length();
      }
      // Get the parameter
      parameter = value.mid( left, ( right - left ) );
      return parameter;
    }
  }
  return QString::null;
}



// Get a value given a field
const QString& MimeMessage::getValue(const QString& field ) const
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Checking for '" << field << "'.  The number of fields is " << fields_.count() << "." << endl;
#endif
  // Search the fields for a match
  for ( uint i = 0; i < fields_.count(); i++ )
  {
#ifdef KMESSDEBUG_MIMEMESSAGE
    kdDebug() << "MimeMessage - Checking does '" << fields_[i] << "' = '" << field << "'?" << endl;
#endif
    if ( fields_[i] == field )
    {
#ifdef KMESSDEBUG_MIMEMESSAGE
      kdDebug() << "MimeMessage - It does." << endl;
#endif

      return values_[i];
    }
  }
  kdDebug() << "MimeMessage - WARNING - This message contained no field \"" << field << "\"." << endl;
  return QString::null;
}



// Test whether a given field exists in the message header
bool MimeMessage::hasField(const QString& field) const
{
  // Search the fields for a match
  for ( uint i = 0; i < fields_.count(); i++ )
  {
    if ( fields_[i] == field )
    {
      return true;
    }
  }

  return false;
}


// Parse the message into type, body, and fields and values
void MimeMessage::parseMessage(const QString& message)
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Parse the message." << endl;
  kdDebug() << "MimeMessage - message:" << endl << message << endl;
#endif
  QString      head, field, value;
  QStringList  lines;
  // Split the message into head and body
  splitMessage( head, body_, message );
  // Split the head into its various lines
  splitHead( lines, head );
  // Split all the lines into field and value
  for ( uint i = 0; i < lines.count(); i++ )
  {
    splitLine( field, value, lines[i] );
    if ( !field.isEmpty() )
    {
      // Add the fields and values to the lists
      addField( field, value );
    }
  }
#ifdef KMESSTEST
  ASSERT( fields_.count() == values_.count() );
#endif
}



// Print the contents of the message to kdDebug (for debugging purposes)
void MimeMessage::print() const
{
  kdDebug() << "MimeMessage - Printing MIME message, " << fields_.count() << " fields." << endl;
  // Get the fields and values
  for ( uint i = 0; i < fields_.count(); i++ )
  {
    kdDebug() << "MimeMessage - Print - Field: \"" << fields_[i] << "\" value: \"" << values_[i] << "\"" << endl;
  }
  kdDebug() << "MimeMessage - Print - Body:" << endl << body_ << endl;
}



// Set the message body
void MimeMessage::setBody(const QString& body)
{
  body_ = body;
}



// Split a line between field and value
void MimeMessage::splitLine(QString& field, QString& value, const QString& line) const
{
  int index;
  index = line.find(":");
  if ( index >= 0 )
  {
    field = line.left( index );
    if ( ( index + 1 ) < (int)line.length() )
    {
      value = line.right( line.length() - index - 2 );
    }
    else
    {
      value = QString::null;
    }
  }
  else
  {
    kdDebug() << "MimeMessage: WARNING - Couldn't split line " << line << endl;
  }
}



// Split the message head into components and store it in the string list
void MimeMessage::splitHead(QStringList& stringList, const QString& head) const
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Splitting the head." << endl;
#endif
  int left = 0, right = 0;
  QString line;

  stringList.clear();
  right = head.find( "\r\n" );
  while ( right >= 0 )
  {
#ifdef KMESSDEBUG_MIMEMESSAGE
    kdDebug() << "MimeMessage - There is an /r/n at " << right << "." << endl;
#endif
    // Get the message between "left" and "right".
    line = head.mid( left, right - left );
    // The new left is the old right + 2
    left = right + 2;
    // The new right is the next occurence of "\r\n" after left.
    right = head.find( "\r\n", left );
    // Add the line to the string list
    stringList << line;
  }
}



// Split a message into head and body
void MimeMessage::splitMessage(QString& head, QString& body, const QString& message) const
{
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - Splitting the message." << endl;
#endif
  int index;

  // The message is split into head and body at "\r\n\r\n"
  index = message.find( "\r\n\r\n" );
#ifdef KMESSDEBUG_MIMEMESSAGE
  kdDebug() << "MimeMessage - The /r/n/r/n  separator is at " << index << "." << endl;
#endif
  if ( index < 0 )
  {
    head = message;
    body = "";
  }
  else
  {
    head = message.left( index + 2 ); // Keep a "\r\n" at the end
    body = message.right( message.length() - index - 4 );
  }
}
