/***************************************************************************
                          msnswitchboardconnection.cpp  -  description
                             -------------------
    begin                : Fri Jan 24 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "msnswitchboardconnection.h"

#include <qregexp.h>

#include <kdebug.h>
#include <kextendedsocket.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "../contact/contactlist.h"
#include "../currentaccount.h"
#include "../kmessdebug.h"
#include "applications/application.h"
#include "applications/gnomemeeting.h"
#include "applications/msnremotedesktop.h"
#include "applications/kderemotedesktop.h"
#include "applications/receivefile.h"
#include "applications/sendfile.h"
#include "applications/transferfile.h"
#include "chatinformation.h"
#include "mimemessage.h"

#ifdef KMESSDEBUG_SWITCHBOARD
  #define KMESSDEBUG_SWITCHBOARD_GENERAL
//  #define KMESSDEBUG_SWITCHBOARD_DESKTOP
//  #define KMESSDEBUG_SWITCHBOARD_FILETRANSFER
//  #define KMESSDEBUG_SWITCHBOARD_MEETING
  #define KMESSDEBUG_SWITCHBOARD_MESSAGE_LENGTH
#endif

// The constructor
MsnSwitchboardConnection::MsnSwitchboardConnection()
 : MsnConnection("SWITCHBOARD"),
   chatInfo_(0),
   contactsConnected_(false),
   initialized_(false)
{
}



// The destructor
MsnSwitchboardConnection::~MsnSwitchboardConnection()
{
}



// Add a contact to the list of contacts in the chat
void MsnSwitchboardConnection::addContactToList(const QString& handle)
{
  // Add the contact to the list if the contact isn't there already
  if ( !isContactInChat( handle ) )
  {
    contactsInChat_.append( handle );
  }
}



// Close the connection
void MsnSwitchboardConnection::closeConnection()
{
  // Send a "bye"
  sendCommand( "BYE", "\r\n");
  disconnectFromServer();
}



// Make the neccessary application connections.
void MsnSwitchboardConnection::connectApplication(Application *app)
{
  connect( app,  SIGNAL(             deleteMe(QString) ),
           this,   SLOT( terminateApplication(QString) ) );
  connect( app,  SIGNAL(        messageToUser(QString) ),
           this,   SLOT(    forwardAppMessage(QString) ) );
  connect( app,  SIGNAL(               putMsg(QString) ),
           this,   SLOT(            putAppMsg(QString) ) );

  applications_.append( app );
}



// The socket connected, so send the version command.
void MsnSwitchboardConnection::connectionSuccess()
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard: Connected Asynchronously, so send VER." << endl;
#endif
  if ( chatInfo_->getUserStartedChat() )
  { // This is a user-initiated chat
    // Set the usr information to the server.
    sendCommand( "USR", currentAccount_->getHandle() + " " + chatInfo_->getAuthorization() + "\r\n" );
  }
  else
  { // This is a contact-initiated chat
    // Answer the chat with the authorization
    sendCommand( "ANS", currentAccount_->getHandle() + " " + chatInfo_->getAuthorization() + " " + chatInfo_->getChatId() + "\r\n" );
  }
}



// Do what's required when a contact comes online
void MsnSwitchboardConnection::contactOnline(QString& handle, QString& friendlyName)
{
  contactsConnected_ = true;
  removePercents( friendlyName );
  addContactToList( handle );
  sendPendingMessages();
  emit contactJoinedChat( handle, friendlyName );
}



// Convert an html format (#RRGGBB) color to an msn format (BBGGRR) color
void MsnSwitchboardConnection::convertHtmlColorToMsnColor(QString &color) const
{
#ifdef KMESSTEST
  ASSERT( color.length() == 7 );
#endif
  // Get the color components
  QString red   = color.mid(1, 2);
  QString green = color.mid(3, 2);
  QString blue  = color.mid(5, 2);

  // Reassemble the color
  if ( blue != "00" )
  {
    color = blue + green + red;
  }
  else if ( green != "00" )
  {
    color = green + red;
  }
  else if ( red != "00" )
  {
    color = red;
  }
  else
  {
    color = "0";
  }
#ifdef KMESSTEST
  ASSERT( color.length() < 7 );
#endif
}



// Convert and msn format color (BBGGRR) to an html format (#RRGGBB) color
void MsnSwitchboardConnection::convertMsnColorToHtmlColor(QString &color) const
{
#ifdef KMESSTEST
  ASSERT( color.length() < 7 );
#endif
  if ( color == "0" )
  {
    color = "#000000";
  }
  else
  {
    // Fill the color out to six characters
    while ( color.length() < 6 )
    {
      color = "0" + color;
    }
    // Get the color components
    QString blue  = color.mid(0, 2);
    QString green = color.mid(2, 2);
    QString red   = color.mid(4, 2);
    // Reassemble the components
    color = "#" + red + green + blue;
  }
#ifdef KMESSTEST
  ASSERT( color.length() == 7 );
#endif
}



// Create an application of the given type
Application* MsnSwitchboardConnection::createApplication(ApplicationType type)
{
  Application *app = 0;

  if ( type == GNOMEMEETING )
  {
    app = new GnomeMeeting( getLocalIp() );
  }
  else if ( type == MSNREMOTEDESKTOP )
  {
    app = new MsnRemoteDesktop( getLocalIp() );
  }
  else if ( type == KDEREMOTEDESKTOP )
  {
    app = new KdeRemoteDesktop( getLocalIp() );
  }
  else if ( type == SENDFILE )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - Making 'send file'" << endl;
#endif
    app = new SendFile( CurrentAccount::instance()->getHandle(), getLocalIp() );
  }
  else if ( type == RECEIVEFILE )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - Making 'receive file'" << endl;
#endif
    app = new ReceiveFile(CurrentAccount::instance()->getHandle(), getLocalIp() );
  }
  if ( app != 0 )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - Connecting the application." << endl;
#endif
    // Connect up the application
    connect( app,  SIGNAL(             deleteMe(QString) ),
             this,   SLOT( terminateApplication(QString) ) );
    connect( app,  SIGNAL(        messageToUser(QString) ),
             this,   SLOT(    forwardAppMessage(QString) ) );
    connect( app,  SIGNAL(               putMsg(QString) ),
             this,   SLOT(            putAppMsg(QString) ) );
    // Add it to the list of applications
    applications_.append( app );
  }
  return app;
}



// Forward an application message by reemitting it as a signal.
void MsnSwitchboardConnection::forwardAppMessage(QString message)
{
  emit showAppMessage( message );
}



// Return the application with the given cookie.
Application* MsnSwitchboardConnection::getApplication(QString cookie) const
{
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "Switchboard::getApplication( cookie = " << cookie << " )" << endl;
#endif
  Application                   *application = 0;
  QPtrListIterator<Application>  it(applications_);

  while ( it.current() != 0 )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "SC:   Does " << it.current()->getCookie() << " = " << cookie << " ?" << endl;
#endif
    if ( it.current()->getCookie() == cookie )
    {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
      kdDebug() << "SC:   Found the application.  Save it and break the loop." << endl;
#endif
      application = it.current();
      break;
    }
    ++it;
  }
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "SC:   Done getApplication(cookie)." << endl;
#endif
  return application;
}



// Get the switchboard to make a list of the contacts in the chat
QString MsnSwitchboardConnection::getCaption()
{
  QString      caption;
  QString     contactString;
  ContactList *contactList;

  if ( chatInfo_ != 0 )
  {
    contactList = chatInfo_->getContactList();
    if ( contactList != 0 )
    {
      // Make a list of the contacts in the chat
      if ( contactsInChat_.count() == 1 )
      {
        contactString = " - " + contactList->getContactFriendlyNameByHandle(contactsInChat_[0]);
      }
      else if ( contactsInChat_.count() == 2 )
      {
        contactString = " - " + contactList->getContactFriendlyNameByHandle( contactsInChat_[0] )
                       + i18n(" and ") + contactList->getContactFriendlyNameByHandle( contactsInChat_[1] );
      }
      else if ( contactsInChat_.count() > 2 )
      {
        contactString = " - " + i18n("%1 et al.").arg(  contactList->getContactFriendlyNameByHandle( contactsInChat_[0] ) );
      }
      else
      {
        contactString = "";
      }
      caption = i18n("Chat") + contactString;
      return caption;
    }
  }
  return i18n("Chat");
}



// Get a contact's friendly name from the contact list
QString MsnSwitchboardConnection::getContactFriendlyNameByHandle(const QString &handle)
{
  if ( chatInfo_ == 0 )
  {
    kdDebug() << "Switchboard::getContactFriendlyNameByHandle(" << handle <<") - WARNING - Chat info is null." << endl;
    return "";
  }
  ContactList *contactList = chatInfo_->getContactList();
  if ( contactList == 0 )
  {
    kdDebug() << "Switchboard::getContactFriendlyNameByHandle(" << handle <<") - WARNING - Contact list is null." << endl;
    return "";
  }
  QString friendlyName = contactList->getContactFriendlyNameByHandle( handle );
  return friendlyName;
}



// Make a list of the contacts in the chat
QStringList MsnSwitchboardConnection::getContactList() const
{
  QStringList                       contacts;
  QValueListConstIterator<QString> it;

  for ( it = contactsInChat_.begin(); it != contactsInChat_.end(); ++it )
  {
    contacts << (*it);
  }

  return contacts;
}



// Get the font and color from a message
void MsnSwitchboardConnection::getFontAndColorFromMessage(QFont &font, QString &color, const MimeMessage &message) const
{
  // Get the font from the format string
  getFontFromMessageFormat( font, message );
  // Get the font color from the message format
  getFontColorFromMessageFormat( color, message );
}



// Get a font from the message
void MsnSwitchboardConnection::getFontFromMessageFormat(QFont &font, const MimeMessage& message) const
{
  QString family  = message.getSubValue("X-MMS-IM-Format", "FN");
  QString effects = message.getSubValue("X-MMS-IM-Format", "EF");

  removePercents( family );
  font.setFamily( family );
  if ( effects.contains("B") )
    font.setBold(true);
  else
    font.setBold(false);
  if ( effects.contains("I") )
    font.setItalic(true);
  else
    font.setItalic(false);
  if ( effects.contains("U") )
    font.setUnderline(true);
  else
    font.setUnderline(false);
}



// Get an html font color from the message
void MsnSwitchboardConnection::getFontColorFromMessageFormat(QString &color, const MimeMessage& message) const
{
  color = message.getSubValue("X-MMS-IM-Format", "CO");
  convertMsnColorToHtmlColor( color );
}



// Extract some information from a font message (i.e. "EF=UIB;" extract the "UIB" )
QString MsnSwitchboardConnection::getFontValueFromMessageFormat(const QString& field, const QString& message) const
{
#ifdef KMESSTEST
  ASSERT( message.contains( field ) );
#endif
  // Look for "field=" in the message...
  int index = message.find( field );
  if ( index < 0 )
  {
    // The field wasn't found
    return "";
  }
  // Move the index ahead past the field and the = that should follow it
  index += field.length() + 1;
  // Get everything to the right of the index
  QString value = message.right( message.length() - index );
  // Find the first ';' in the value, which should mark then end of the value
  value = value.left( value.find(";") );
  return value;
}



// Return a properly created message head
QString MsnSwitchboardConnection::getMessageHead()
{
  if ( currentAccount_ == 0 )
  {
    kdDebug() << "Switchboard::getMessageHead() - WARNING - currentAccount_ is null!" << endl;
    return "";
  }
  QFont   font = currentAccount_->getFont();
  QString fontFamily = font.family();
  addPercents( fontFamily );
  QString color = currentAccount_->getFontColor();
  convertHtmlColorToMsnColor( color );
  QString effects = "";
  if ( font.bold() )      effects += "B";
  if ( font.italic() )    effects += "I";
  if ( font.underline() ) effects += "U";
  QString head  = "MIME-Version: 1.0\r\n";
          head += "Content-Type: text/plain; charset=UTF-8\r\n";
          head += "X-MMS-IM-Format: FN=" + fontFamily + "; EF=" + effects + "; CO=" + color + "; CS=0; PF=0\r\n\r\n";
  return head;
}



// Return the "user is typing" message
QString MsnSwitchboardConnection::getTypingMessage() const
{
#ifdef KMESSTEST
  ASSERT( currentAccount_ != 0 );
#endif
  QString message;
  message = "MIME-Version: 1.0\r\nContent-Type: text/x-msmsgscontrol\r\nTypingUser: " + currentAccount_->getHandle() + "\r\n\r\n";
  return message;
}



// Give the application with the given cookie the command.
void MsnSwitchboardConnection::giveAppCommand(QString cookie, QString command)
{
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "Switchboard: Give app cookie = " << cookie << " command = " << command << "." << endl;
#endif
  Application *application;
  // Find the application indicated by the cookie
  application = getApplication( cookie );
  // If the application wasn't found
  if ( application != 0 )
  {
    application->gotCommand( command );
  }
  else
  {
    kdDebug() << "MsnSwitchboardConnection: WARNING - Application " << cookie << " wasn't found to give command " << command << " to." << endl;
  }
}



// Received a positive delivery message.
void MsnSwitchboardConnection::gotAck(const QStringList& command)
{
#ifdef KMESSTEST
  ASSERT( acks_.count() == messages_.count() );
  uint oldAckCount = acks_.count();
  uint oldMessageCount = messages_.count();
#endif
  QValueList<QString>::iterator ackIt, messageIt;

  QString ack = command[1];

#ifdef KMESSTEST
  bool goodNumber;
  ack.toUInt( &goodNumber );
  ASSERT( goodNumber );
#endif
  messageIt = messages_.begin();
  for ( ackIt = acks_.begin(); ackIt != acks_.end(); ++ackIt )
  {
    if ( (*ackIt) == ack )
    {
      acks_.remove( ackIt );
      messages_.remove( messageIt );
      break;
    }
    ++messageIt;
  }
#ifdef KMESSTEST
  ASSERT( acks_.count() == (oldAckCount - 1) );
  ASSERT( messages_.count() == (oldMessageCount - 1) );
#endif
}



// Received notification that a contact is no longer in session.
void MsnSwitchboardConnection::gotBye(const QStringList& command)
{
  removeContactFromList( command[1] );
  emit contactLeftChat( command[1] );
}



// Received the initial roster information for new contacts joining a session.
void MsnSwitchboardConnection::gotIro(const QStringList& command)
{
  QString handle       = command[4];
  QString friendlyName = command[5];
  removePercents( friendlyName );

  if ( chatInfo_ == 0 )
  {
    kdDebug() << "Switchboard - gotIro - WARNING - ChatInfo is null!" << endl;
  }
  else
  {
    QString altFriendlyName = chatInfo_->getContactList()->getContactFriendlyNameByHandle( handle );
    if ( altFriendlyName != "" )
    {
      friendlyName = altFriendlyName;
    }
  }
  contactOnline( handle, friendlyName );
}



// Received notification of a new client in the session.
void MsnSwitchboardConnection::gotJoi(const QStringList& command)
{
  QString handle       = command[1];
  QString friendlyName = command[2];
  removePercents( friendlyName );

  if ( chatInfo_ == 0 )
  {
    kdDebug() << "Switchboard - gotIro - WARNING - ChatInfo is null!" << endl;
  }
  else
  {
    QString altFriendlyName = chatInfo_->getContactList()->getContactFriendlyNameByHandle( handle );
    if ( altFriendlyName != "" )
    {
      friendlyName = altFriendlyName;
    }
  }
  contactOnline( handle, friendlyName );
}



// Received a negative acknowledgement of the receipt of a message.
void MsnSwitchboardConnection::gotNak(const QStringList& command)
{
#ifdef KMESSTEST
  ASSERT( acks_.count() == messages_.count() );
  uint oldAckCount = acks_.count();
  uint oldMessageCount = messages_.count();
#endif
  QValueList<QString>::iterator ackIt, messageIt;

  QString ack = command[1];

#ifdef KMESSTEST
  bool goodNumber;
  ack.toUInt( &goodNumber );
  ASSERT( goodNumber );
#endif
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - NAK number is " << ack << endl;
#endif
  messageIt = messages_.begin();
  for ( ackIt = acks_.begin(); ackIt != acks_.end(); ++ackIt )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
    kdDebug() << "Switchboard - Have stored ack of " << (*ackIt) << endl;
#endif
    if ( (*ackIt) == ack )
    {
      QString message = i18n("The message \"%1\" was not received").arg( *messageIt );
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
      kdDebug() << "Switchboard - Give the message: " << message << endl;
#endif
      emit systemMessage( message );
      acks_.remove( ackIt );
      messages_.remove( messageIt );
      break;
    }
    ++messageIt;
  }
#ifdef KMESSTEST
  ASSERT( acks_.count() == (oldAckCount - 1) );
  ASSERT( messages_.count() == (oldMessageCount - 1) );
#endif
}



// Received notification of the termination of a client-server session.
void MsnSwitchboardConnection::gotOut(const QStringList& /*command*/)
{
/* Don't do anything, I guess
  closeConnection();
*/
}



// Received a client-server authentication message.
void MsnSwitchboardConnection::gotUsr(const QStringList& command)
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - got USR." << endl;
#endif
#ifdef KMESSTEST
  ASSERT( chatInfo_ != 0 );
#endif
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - Contact handle is " << chatInfo_->getContactHandle() << endl;
#endif
  // This should just be a confirmation
  if ( command[2] != "OK" )
  {
    kdDebug() << "MsnSwitchboardConnection: ERROR: Switchboard authentication failed" << endl;
    return;
  }
  if ( chatInfo_ == 0 )
  {
    kdDebug() << "Switchboard - gotUsr - WARNING - ChatInfo is null!" << endl;
  }
  else
  {
    // Now call the other user to the conversation.
    sendCommand( "CAL", chatInfo_->getContactHandle() + "\r\n" );
  }
}



// Initialize the object
bool MsnSwitchboardConnection::initialize()
{
  if ( initialized_ )
  {
    kdDebug() << "Notification Connection already initialized!" << endl;
    return false;
  }
  if ( !MsnConnection::initialize() )
  {
    kdDebug() << "Notification Connection: Couldn't initialize parent." << endl;
    return false;
  }
  initialized_ = true;
  return initialized_;
}



// Invite a contact into the chat
void MsnSwitchboardConnection::inviteContact(QString handle)
{
  if ( handle != "" )
  {
    sendCommand("CAL", handle + "\r\n");
  }
}



// Check if a certain contact is in the chat
bool MsnSwitchboardConnection::isContactInChat(const QString& handle) const
{
  QValueListConstIterator<QString> it;

  for ( it = contactsInChat_.begin(); it != contactsInChat_.end(); ++it )
  {
    if ( (*it) == handle )
    {
      return true;
    }
  }
  return false;
}



// Check if only the given contact is in the chat
bool MsnSwitchboardConnection::isExclusiveChatWithContact(const QString& handle) const
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - Checking if chat is exclusive with " << handle << "." << endl;
  kdDebug() << "Switchboard - There are " << contactsInChat_.count() << " contacts in the chat." << endl;
#endif
  if ( ( contactsInChat_.count() == 1 ) && ( isContactInChat( handle ) ) )
  {
    return true;
  }
  return false;
}



// Parse an application message
void MsnSwitchboardConnection::parseAppMessage(const MimeMessage &message)
{
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "Switchboard - Parse the application message." << endl;
  message.print();
#endif
  Application *app = 0;

  // If this is an invite message, an application need to be created
  if ( message.getValue("Invitation-Command") == "INVITE" )
  {
    // Create the application based on the type
    QString type = message.getValue("Application-GUID");
    if ( type == GnomeMeeting::getAppId() )
    {
      app = createApplication( GNOMEMEETING );
    }
    else if ( type == MsnRemoteDesktop::getAppId() )
    {
      app = createApplication( MSNREMOTEDESKTOP );
    }
    else if ( type == KdeRemoteDesktop::getAppId() )
    {
      app = createApplication( KDEREMOTEDESKTOP );
    }
    else if ( type == TransferFile::getAppId() )
    {
      app = createApplication( RECEIVEFILE );
    }
    else
    {
      // The type of the application wasn't recognized, so reject it out of hand
      QString warning = i18n("The contact is inviting you to start %1, but this is not implemented yet.").arg( message.getValue("Application-Name") );
      emit systemMessage( warning );
      rejectApplication( message.getValue("Invitation-Cookie") );
    }
  }
  else
  {
    // Find the application based on the app cookie
    app = getApplication( message.getValue("Invitation-Cookie")  );
  }
  // Give the message to the application
  if ( app != 0 )
  {
    app->gotMessage( message );
  }
}



// Parse a regular command
void MsnSwitchboardConnection::parseCommand(const QStringList& command)
{
  if ( command[0] == "ACK" )
  {
    gotAck( command );
  }
  else if ( command[0] == "ANS" )
  {
    // Do nothing.
  }
  else if ( command[0] == "BYE" )
  {
    gotBye( command );
  }
  else if ( command[0] == "CAL" )
  {
    // Do nothing
  }
  else if ( command[0] == "IRO" )
  {
    gotIro( command );
  }
  else if ( command[0] == "JOI" )
  {
    gotJoi( command );
  }
  else if ( command[0] == "NAK" )
  {
    gotNak( command );
  }
  else if ( command[0] == "OUT" )
  {
    gotOut( command );
  }
  else if ( command[0] == "USR" )
  {
    gotUsr( command );
  }
  else if ( command[0] == "216" )
  {
    emit systemMessage( i18n( "This person is online, but he or she is blocking you." ) );
  }
  else if ( command[0] == "217" )
  {
    emit systemMessage( i18n( "This person is offline or invisible." ) );
  }
  else
  {
    kdDebug() << "Switchboard got unhandled command " << command[0] << "." << endl;
  }
}



// Parse a message command
void MsnSwitchboardConnection::parseMessage(const QStringList& command, const MimeMessage &message)
{
  // Get the message type from the head
  QString contentType = message.getSubValue( "Content-Type" );
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard: Message type is '" << contentType << "'." << endl;
#endif
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - Got message type " << contentType << "." << endl;
#endif

  // Get the sender's handle and friendly name
  QString handle, friendlyName;
  handle = command[1];
  // Check the contact list for a more current friendly name
  friendlyName = getContactFriendlyNameByHandle( handle );
  if ( friendlyName == "" )
  {
    // There was no current friendly name, so get one from the message
    friendlyName = command[2];
    removePercents( friendlyName );
  }

  if ( contentType == "text/plain" )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
    kdDebug() << "Switchboard: Message is:" << endl;
    message.print();
#endif
    // This is a regular text message to the user
    QFont    font;
    QString color;
    getFontAndColorFromMessage( font, color, message );

    // Emit the message
    emit chatMessage( handle, friendlyName, message.getBody(), font, color, "#D60000" );
  }
  else if ( contentType == "text/x-msmsgscontrol" )
  {
    QString typingUser = message.getValue( "TypingUser" );
    // Check the contact list for a more current friendly name
    QString typingName = getContactFriendlyNameByHandle( typingUser );
    if ( typingName == "" )
    {
      typingName = typingUser;
    }
    emit contactTyping( typingUser, typingName );
  }
  else if ( contentType == "text/x-msmsgsinvite" )
  {
    MimeMessage subMessage( message.getBody() );
    // This is an application message
    parseAppMessage( subMessage );
  }
}



// Send an application message to the server.
void MsnSwitchboardConnection::putAppMsg(QString message)
{
  // Simply send the message on to putMsg.  As long as the applications
  //  can be trusted to send correctly formatted data, no checking needs to
  //  be done.
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "Switchboard - Putting app message" << endl;
#endif
  QString command;
  command.sprintf("N %u\r\n", message.utf8().length() );
  sendCommand( "MSG", command + message );
}



// Send a reject message for the given application
void MsnSwitchboardConnection::rejectApplication(const QString& cookie)
{
  QString command, message;
  message  = "MIME-Version: 1.0\r\n";
  message += "Content-Type: text/x-msmsgsinvite; charset=UTF-8\r\n\r\n";
  message += "Invitation-Command: CANCEL\r\n";
  message += "Invitation-Cookie: " + cookie + "\r\n";
  message += "Cancel-Code: REJECT";

  command.sprintf( "N %u\r\n", message.utf8().length() );
  sendCommand( "MSG", command + message );
}



// Remove a contact from the list of contacts in the chat
void MsnSwitchboardConnection::removeContactFromList(const QString& handle)
{
  QValueList<QString>::iterator it;

  // Check if the contact is in the chat..
  if ( isContactInChat( handle ) )
  {
    // Leave at least one contact in the chat
    if ( contactsInChat_.count() > 1 )
    {
      // Find the contact in the chat
      for ( it = contactsInChat_.begin(); it != contactsInChat_.end(); ++it )
      {
        if ( (*it) == handle )
        {
          contactsInChat_.remove( it );
          break;
        }
      }
    }
    else
    {
      // There is only one contact in the chat and it wasn't removed in order to have a default.
      contactsConnected_ = false;
    }
  }
}



// Send a message to the contact(s)
void MsnSwitchboardConnection::sendMessage(QString text)
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - Sending message." << endl;
#endif
  // Check if there are any contacts connected
  if ( contactsConnected_ )
  {
    // There are contacts in the chat, so send the message normally.
    int ack;

    QString head = getMessageHead();
    QString message = head + text;
    QString command;
    command.sprintf( "A %u\r\n", message.utf8().length() );
#ifdef KMESSDEBUG_SWITCHBOARD_MESSAGE_LENGTH
    kdDebug() << "Message command: " << command << endl;
#endif
    ack = sendCommand( "MSG", command + message );
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
    kdDebug() << "Switchboard - Command + message is '" << command + message <<"'." << endl;
#endif
    // Save the message acknowledgement
    storeMessageForAcknowledgement( ack, text );
  }
  else
  {
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
    kdDebug() << "Switchboard - No contacts are presently in the chat - call the one kept." << endl;
#endif
    // Store the message
    pendingMessages_.append( text );
    // Call the first (should be only) contact in the "contacts in chat" list.
#ifdef KMESSTEST
    ASSERT( contactsInChat_.count() == 1 );
#endif
    if ( contactsInChat_.count() > 0 )
    {
      QString contactToCall = contactsInChat_.first();
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
      kdDebug() << "Switchboard - Call " << contactToCall << "." << endl;
#endif
      sendCommand( "CAL", contactToCall + "\r\n");
    }
  }
}



// Send messages that weren't sent because a contact had to be re-called
void MsnSwitchboardConnection::sendPendingMessages()
{
  QValueList<QString>::iterator it;

#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  if ( pendingMessages_.count() > 0 )
  {
    kdDebug() << "Switchboard - Sending " << pendingMessages_.count() << " pending messages." << endl;
  }
#endif
  // A contact should be connected, but check to make sure
  if ( contactsConnected_ )
  {
    // Go through the pending messages
    for ( it = pendingMessages_.begin(); it != pendingMessages_.end(); ++it )
    {
      // Re-send each one.
      sendMessage( (*it ) );
    }
    // Clear the pending messages.
    pendingMessages_.clear();
#ifdef KMESSTEST
    ASSERT( pendingMessages_.count() == 0 );
#endif
  }
}



// Show a "Cancelled" message to the user
void MsnSwitchboardConnection::showAppCancelled()
{
  QString html;
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "SC:   The transfer was cancelled." << endl;
#endif
  // Create a cancel message for the user.
  html  = "<font color=purple>";
  html += i18n("Cancelled");
  html += ".</font><br><br>";
  emit showAppMessage( html );
}



// Start an application
void MsnSwitchboardConnection::startApp(ApplicationType type)
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - start application " << type << "." << endl;
#endif
  Application *app;

  // Create the application
  app = createApplication( type );
  // If it was created...
  if ( app != 0 )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - Starting the application." << endl;
#endif
    // Start it
    app->start();
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "Switchboard - The application's cookie is " << app->getCookie() << endl;
#endif
  }
}



// Start a chat
void MsnSwitchboardConnection::startChat(ChatInformation *chatInfo)
{
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - starting chat." << endl;
#endif
  if ( chatInfo == 0 )
  {
    kdDebug() << "Switchboard given bad chat information." << endl;
    return;
  }
#ifdef KMESSTEST
  ASSERT( currentAccount_ != 0 );
  ASSERT( chatInfo != 0 );
  ASSERT( chatInfo->getContactList() != 0 );
  ASSERT( chatInfo->getContactHandle() != 0 );
#endif
  chatInfo_ = chatInfo;

#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - The chat is with " << chatInfo_->getContactHandle() << "." << endl;
#endif
  // Disconnect if connected
  closeConnection();
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
  kdDebug() << "Switchboard - Connect to " << chatInfo_->getIp() << ":" << chatInfo_->getPort() << "." << endl;
#endif
  // The chat is either user-started (in which case there is no ID) or
  //  contact started, in which case the ID is information to be passed
  //  to the switchboard server -- but all that will be determined at
  //  connectionSuccess.  For now, connect to the server.
  if ( !connectToServer( chatInfo_->getIp(), chatInfo_->getPort() ) )
  {
    kdDebug() << "Switchboard couldn't connect to the server." << endl;
  }

/*
  // The chat is either user-started (in which case there is no ID) or
  //  contact started, in which case the ID is information to be passed
  //  to the switchboard server
  if ( chatInfo_->getUserStartedChat() )
  { // This is a user-initiated chat
    // Open a connection to the server
    if ( connectToServer( chatInfo_->getIp(), chatInfo_->getPort() ) )
    {
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
      kdDebug() << "Switchboard - Sending USR confirmation." << endl;
#endif
      // Set the usr information to the server.
      sendCommand( "USR", currentAccount_->getHandle() + " " + chatInfo_->getAuthorization() + "\r\n" );
    }
    else
    {
      kdDebug() << "Switchboard couldn't connect to the server." << endl;
    }
  }
  else
  { // This is a contact-initiated chat
    // Open a connection to the server
    if ( connectToServer( chatInfo_->getIp(), chatInfo_->getPort() ) )
    {
#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
      kdDebug() << "Switchboard - Sending ANS confirmation." << endl;
#endif
      // Answer the chat with the authorization
      sendCommand( "ANS", currentAccount_->getHandle() + " " + chatInfo_->getAuthorization() + " " + chatInfo_->getChatId() + "\r\n" );
    }
    else
    {
      kdDebug() << "Switchboard couldn't connect to the server." << endl;
    }
  }
*/
}



// Store a message for later acknowledgement
void MsnSwitchboardConnection::storeMessageForAcknowledgement(const int& ack, const QString& text)
{
#ifdef KMESSTEST
  uint ackCount = acks_.count();
  uint messageCount = messages_.count();
#endif
  QString ackString;

  ackString.sprintf("%d", ack);
  acks_.append( ackString );
  messages_.append( text );

#ifdef KMESSTEST
  ASSERT( acks_.count() == (ackCount + 1) );
  ASSERT( messages_.count() == (messageCount + 1) );
#endif
}



// Remove and delete the given application.
void MsnSwitchboardConnection::terminateApplication(QString cookie)
{
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "MsnSwbdCnxn::terminateApplication( cookie = " << cookie << " )" << endl;
#endif
  Application *application;

#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "SC:   Deleting application." << endl;
#endif
  // Find the indicated application
  application = getApplication( cookie );
  // If the application was found..
  if ( application != 0 )
  {
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "SC:   Remove the app from the list." << endl;
#endif
    // Remove the application from the list.
    applications_.remove( application );
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
    kdDebug() << "SC:   Delete the app." << endl;
#endif
    // Delete the application
    delete application;
  }
  else
  {
    kdDebug() << "MsnSwitchboardConnection: WARNING - Application " << cookie
              << " couldn't be found to delete." << endl;
  }
#ifdef KMESSDEBUG_SWITCHBOARD_FILETRANSFER
  kdDebug() << "SC:   Done terminateApplication(cookie)." << endl;
#endif
}



// The user is typing so send a typing message
void MsnSwitchboardConnection::userIsTyping()
{
  QString command;
  QString message = getTypingMessage();
  // Append the message length to the command, plus two for the /r/n that will go on the end.
  command.sprintf("U %u\r\n", message.utf8().length());
  sendCommand("MSG", command + message);
}
