/***************************************************************************
                          sslloginhandler.cpp  -  description
                             -------------------
    begin                : Sat Jun 28 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sslloginhandler.h"

#include <qstringlist.h>
#include <qregexp.h>
#include <qsocket.h>

#include <kdebug.h>
#include <kextsock.h>
#include <kssl.h>
#include <kurl.h>

#include "../kmessdebug.h"
#include "mimemessage.h"

// The constructor
SslLoginHandler::SslLoginHandler()
 : mode_(NONE)
{
  // Clear cookie list
  cookies_.clear();
  // Create the SSL handler
  ssl_ = new KSSL( true );
  // Create and set up the socket.
  socket_ = new KExtendedSocket( );
  //socket_->setSocketFlags( 0x00 | 0x600000 ); // 0x00 = anySocket | 0x600000 = bufferedSocket
  socket_->setSocketFlags( 0x00 ); // 0x00 = anySocket | 0x600000 = bufferedSocket
  socket_->setTimeout( 30   );
  socket_->enableRead( true );
  connect( socket_, SIGNAL(         readyRead()    ),
           this,    SLOT  (      dataReceived()    ) );
  connect( socket_, SIGNAL(  connectionFailed(int) ),
           this,    SLOT  (       socketError(int) ) );
}



// The destructor
SslLoginHandler::~SslLoginHandler()
{
  delete ssl_;
  delete socket_;
}



// Data was received over the socket
void SslLoginHandler::dataReceived()
{
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - ******************** Data received *******************************" << endl;
  kdDebug() << "SslLoginHandler - " << socket_->bytesAvailable() << " bytes available." << endl;
  kdDebug() << "SslLoginHandler - SSL says " << ssl_->pending() << " bytes available." << endl;
#endif
  QString data;
  int     breakOut = 0, maxIterations = 1000;
  while ( ( !data.contains( QRegExp("\r\n") ) ) && ( breakOut < maxIterations ) )
  {
    // Read data via SSL
    data = readSslData();
    breakOut ++;
  }
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "******************* Read *****************************************************************" << endl
            << data << endl
            << "*****************************************************************************************" << endl;
#endif
  if ( breakOut >= maxIterations )
  {
    kdDebug() << "WARNING - SSL read timed out." << endl;
    emit loginFailed();
  }
  if ( data.length() > 0 )
  {
    // Create a MimeMessage, removing the HTTP status header
    MimeMessage message( data.section( ",", 1 ) );

    if ( message.hasField( "Location" ) )
    {
      QString request;
      KURL   location;

      parseCookieData( message );

      location = KURL( message.getValue( "Location" ) );

      request =   "GET " + location.encodedPathAndQuery() + " HTTP/1.0\r\n";
      for ( uint cookie = 0; cookie < cookies_.count() ; cookie++ )
      {
        request += "Cookie: " + cookies_[ cookie ] + "\r\n";
      }
      request += "\r\n";

      sendHttpRequest( request, location.host(), 443 );

      //dataReceived();
    }
    else if ( mode_ == GETLOGINSERVER )
    {
      QString loginServer, page;
      // This data describes the login server to use.
      parseServerData( loginServer, page, data );
      // Send the cookie request
      sendCookieRequest( loginServer, page );

      //dataReceived();
    }
    else if ( mode_ == GETCOOKIES )
    {
      //QString cookie1, cookie2;
      parseCookieData( message );
      sendAuthenticationRequest( authenticationParameters_, handle_, password_ );
      //dataReceived();
    }
    else if ( mode_ == GETAUTHENTICATIONDATA )
    {
      parseAuthenticationData( data );
    }
  }
  else
  {
    kdDebug() << "WARNING - Available data wasn't read from the SSL socket." << endl;
    emit loginFailed();
  }
//  NONE = 0, GETLOGINSERVER = 1, GETCOOKIES = 2, GETLOGINDATA = 3
}



// Get a login server based on the user's domain
QString SslLoginHandler::getAuthenticationServer( QString &domain )
{
  QString server;

  if ( domain == "hotmail.com" )
  {
    server = "loginnet.passport.com";
    domain = "hotmail.com";
  }
  else if ( domain == "msn.com" )
  {
    server = "msnialogin.passport.com";
    domain = "hotmail.com";
  }
  else if ( domain == "compaq.net" )
  {
    server = "msnialogin.passport.com";
    domain = "passport.com";
  }
  else if ( domain == "passport.com" )
  {
    server = "login.passport.com";
    domain = "passport.com";
  }
  else if ( domain == "webtv.net" )
  {
    server = "login.passport.com";
    domain = "passport.com";
  }
  else if ( domain == "charter.com" )
  {
    server = "login.passport.com";
    domain = "passport.com";
  }
  else
  {
    server = "login.passport.com";
    domain = "";
  }
  return server;
}



// Start the login process
void SslLoginHandler::login( QString parameters, QString handle, QString password )
{
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Starting login with parameters " << parameters << "." << endl;
#endif
  // Get authenitcatino parameters,
  // and replace commas with "&" to make it HTTP compliant
  authenticationParameters_ = parameters.replace( QRegExp( "[,]" ), "&" );
  // Store the given data
  handle_   = handle;
  password_ = password;

  // Get the login server
  sendLoginServerRequest();
  dataReceived();
/*
  // Read data via SSL
  QString data = readSslData();
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "******************* Read *****************************************************************" << endl
            << data << endl
            << "*****************************************************************************************" << endl;
#endif
*/
}



// Get the authentication data from a string
void SslLoginHandler::parseAuthenticationData( QString data )
{
  QString tAndP;

  // Pull t out of the message
  tAndP = data.right( data.length() - data.find( QRegExp( "&t=" ) ) - 1 );
  tAndP = tAndP.left( tAndP.find( QRegExp( "\">" ) ) );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - t and p are " << tAndP << "." << endl;
#endif
  emit loginSucceeded( tAndP );
}



// Get the cookie data from a string
void SslLoginHandler::parseCookieData( MimeMessage &cookieData )
{
  QString field, value;
  uint index;

  for ( index = 0; index < cookieData.getNoFields(); index++ )
  {
    cookieData.getFieldAndValue( field, value, index );

    if ( field == "Set-Cookie" )
    {
      if ( !cookies_.contains( value ) )
      {
        cookies_.append( value );
      }
    }
  }

}



// Get the parameters we need from the parameter list
void SslLoginHandler::parseParameters( QString &lc, QString &id, QString &tw, const QString &parameters )
{
  QStringList parameterList;
  // Split the parameters into a string list so that every entry takes two lines, one for the name and one for the value
  parameterList = QStringList::split( QRegExp("[&=]"), parameters );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
/*
  for ( int i = 0; i < parameterList.count(); i++ )
  {
    kdDebug() << "ParameterList[" << i << "] is " << parameterList[i] << endl;
  }
*/
#endif
  // Get the parameters we're interested in
  lc = parameterList[1];
  id = parameterList[3];
  tw = parameterList[5];
}



// Get login server data from a string
void SslLoginHandler::parseServerData( QString &host, QString &page, QString serverData )
{
  int slashIndex;
  serverData = serverData.right( serverData.length() - serverData.find( "DALogin=" ) - 8 );
  serverData = serverData.left( serverData.find( "," ) );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Host/Page is " << serverData << endl;
#endif
  slashIndex = serverData.find( "/" );
  host = serverData.left( slashIndex );
  page = serverData.right( serverData.length() - slashIndex - 1 );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Host is " << host << " Page is " << page << endl;
#endif
}



// Read data from the socket via SSL
QString SslLoginHandler::readSslData()
{
  char       rawblock[1024];
  QCString   block;
  QString    data;
  int        noBytesRead = 1;
#ifdef KMESSDEBUG_SSLLOGINHANDLER
//  kdDebug() << "SslLoginHandler - Reading data." << endl;
#endif
  data = "";
  // Read data from the SSL socket.
  if ( ssl_ != 0 )
  {
//    while( ( ssl_->pending() > 0 ) && ( noBytesRead > 0 ) )
    while ( /*( socket_->bytesAvailable() > 0 ) && */( noBytesRead > 0 ) )
    {
      noBytesRead = ssl_->read( rawblock, 1024 );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
      kdDebug() << "SslLoginHandler - " << noBytesRead << " bytes read." << endl;
#endif
      block = rawblock;
      block = block.left( noBytesRead );
      data = data + QString::fromUtf8( block );
    }
  }
  return data;
}



// Remove percents from a parameter
void SslLoginHandler::removePercents( QString &word )
{
  // There are only a limited number we need to worry about, here.
  word = word.replace( QRegExp( "%2E" ), "." );
  word = word.replace( QRegExp( "%2F" ), "/" );
  word = word.replace( QRegExp( "%3A" ), ":" );
}



// Send the authenticationn request
void SslLoginHandler::sendAuthenticationRequest( QString parameters, QString handle, QString password )
{
  QString request, lc, id, tw, cbid, domain, server, padding;
  // Get the parameters we need from the parameter list
  parseParameters( lc, id, tw, parameters );
  // Get the domain from the handle
  domain = handle.right( handle.length() - handle.find( QRegExp("@") ) - 1 );
  domain = domain.lower();
  //handle = handle.left( handle.find( QRegExp("@") ) );
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - The user's domain is " << domain << "." << endl;
#endif
  // Get a login server based on the user's domain
  server = getAuthenticationServer( domain );

  // Set the user's country code.
  cbid = "507"; // just stick one in for now
  // Compose the request
  request = "GET /ppsecure/post.srf?lc=" + lc + "&id=" + id + "&tw=" + tw + "&cbid=" + cbid
            + "&da=passport.com" "&login=" + handle + "&domain=" + domain  + "&passwd=" + password
            + "&sec=&mspp_shared=&padding= HTTP/1.0\r\n";

  for ( uint cookie = 0; cookie < cookies_.count() ; cookie++ )
  {
    request += "Cookie: " + cookies_[ cookie ] + "\r\n";
  }
  request += "\r\n";

#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - The authentication request is " << request << "." << endl;
#endif
  mode_ = GETAUTHENTICATIONDATA;
  sendHttpRequest( request, server, 443 );
}



// Do the request that gets the login cookies
void SslLoginHandler::sendCookieRequest( QString host, QString page )
{
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Send the request to " << host << " for page " << page << "." << endl;
#endif
  QString request;
  // Make the http request
  request = "GET /" + page + "?" + authenticationParameters_ + " HTTP/1.0\r\n\r\n";
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - The request is " << request << " to " << host << ".  Send it." << endl;
#endif
  // Change the mode to the next phase...
  mode_ = GETCOOKIES;
  // Send the request to the server
  sendHttpRequest( request, host, 443 );
}



// Send the request to the server
void SslLoginHandler::sendHttpRequest( QString request, QString host, int port )
{
  QString response, responseBody;

  if ( socket_ == 0 )
  {
    kdDebug() << "SslLoginHandler::sendHttpRequest - WARNING - Trying to login using a null socket." << endl;
    return;
  }
#ifdef KMESSDEBUG_SSLLOGINHANDLER
    kdDebug() << "SslLoginHandler - Close and reset the socket." << endl;
#endif
  ssl_->setAutoReconfig( true );
  ssl_->reInitialize();
  socket_->closeNow();
  socket_->reset();

#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Set host = " << host << " port = " << port << endl;
#endif
  socket_->setAddress( host, port );

#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Try to connect." << endl;
#endif
  socket_->lookup();
  int connectionSuccess = socket_->connect();
  if ( connectionSuccess != 0 )
  {
    kdDebug() << "SslLoginHandler::sendHttpRequest - WARNING - Connection failed, giving " << connectionSuccess << endl;
    return;
  }
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Socket connected OK.  Try the SSL connection to fd = " << socket_->fd() << endl;
#endif
  int sslConnectionSuccess = ssl_->connect( socket_->fd() );
  if ( sslConnectionSuccess != 1 )
  {
    kdDebug() << "SslLoginHandler::sendHttpRequest - WARNING - SSL Connection failed, giving " << sslConnectionSuccess << endl;
    return;
  }
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - SSL connected OK.  Send the request " << request << endl;
#endif
  writeSslData( request );
}



// Request the name of the login server
void SslLoginHandler::sendLoginServerRequest()
{
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Request the login server." << endl;
#endif
  QString request;
  // Create the request
  request = "GET /rdr/pprdr.asp\r\n\r\n";
  // Change the mode
  mode_ = GETLOGINSERVER;
#ifdef KMESSDEBUG_SSLLOGINHANDLER
  kdDebug() << "SslLoginHandler - Send the request." << endl;
#endif
  // Get the page
  sendHttpRequest( request, "nexus.passport.com", 443 );
}



// Detect a socket error
void SslLoginHandler::socketError(int error)
{
  kdDebug() << "SslLoginHandler : WARNING - Received error " << error << " from the socket." << endl;
}



// Write data to the socket via SSL
void SslLoginHandler::writeSslData( QString data )
{
  int      noBytesWritten;
  QCString unicode;

  if ( ( socket_ != 0 ) && ( ssl_ != 0 ) )
  {
    noBytesWritten = ssl_->write( data.latin1(), data.length() );
    if ( noBytesWritten != (int)data.length() )
    {
      kdDebug() << "WARNING - Wanted to write " << data.length() << " to the socket, wrote " << noBytesWritten << "." << endl;
    }
#ifdef KMESSDEBUG_SSLLOGINHANDLER
    else
    {
      kdDebug() << "SslLoginHandler - Sent " << noBytesWritten << " bytes via SSL." << endl;
    }
#endif
  }
}

